# Copyright (C) 2003  CAMP
# Please see the accompanying LICENSE file for further information.

import os
import sys

import numpy as npy

from asap3 import _asap

MASTER = 0

def is_contiguous(array, dtype=None):
    """Check for contiguity and type."""
    if dtype is None:
        return array.flags.contiguous
    else:
        return array.flags.contiguous and array.dtype == dtype

# Serial communicator
class SerialCommunicator:
    size = 1
    rank = 0
    def sum(self, array, root=-1):
        if isinstance(array, (float, complex)):
            return array

    def scatter(self, s, r, root):
        r[:] = s

    def max(self, value, root=-1):
        return value

    def broadcast(self, buf, root):
        pass

    def send(self, buff, root, tag=123, block=True):
        pass

    def barrier(self):
        pass

    def gather(self, a, root, b):
        b[:] = a

    def new_communicator(self, ranks):
        return self

    def cart_create(self, dimx, dimy, dimz, periodic):
        return self

class DummyCommunicator(SerialCommunicator):
    size = 1
    size = 0
    def new_communicator(self, ranks):
        new_comm = DummyCommunicator()
        new_comm.size = len(ranks)
        return new_comm


# serial_comm = SerialCommunicator()

if hasattr(_asap, "Communicator"):
    class _Communicator(_asap.Communicator):
        def new_communicator(self, ranks):
            assert is_contiguous(ranks, int)
            sranks = npy.sort(ranks)
            # Are all ranks in range?
            assert 0 <= sranks[0] and sranks[-1] < self.size
            # No duplicates:
            for i in range(len(sranks) - 1):
                assert sranks[i] != sranks[i + 1]
            comm = _asap.Communicator.new_communicator(self, ranks)
            if comm is None:
                # This cpu is not in the new communicator:
                return None
            else:
                # Oops, this will be the original _asap.Communicator class.
                return comm

        def broadcast_string(self, string, root):
            if rank == root:
                if sys.version_info[0] == 2:
                    # Python 2 
                    assert isinstance(string, str)
                else:
                    # Python 3
                    assert isinstance(string, bytes)
                n = npy.array(len(string), int)
            else:
                n = npy.zeros(1, int)
            self.broadcast(n, root)
            if rank == root:
                string = npy.fromstring(string, npy.int8)
            else:
                string = npy.zeros(n, npy.int8)
            self.broadcast(string, root)
            return string.tostring()

    world = _Communicator()
else:
    world = SerialCommunicator()

size = world.size
rank = world.rank
parallel = (size > 1)


def all_gather_array(comm, a): #???
    # Gather array into flat array
    shape = (comm.size,) + npy.shape(a)
    all = npy.zeros(shape)
    comm.all_gather(a, all)
    return all.ravel()


# def run(iterators):
#     """Run through list of iterators one step at a time."""
#     if not isinstance(iterators, list):
#         # It's a single iterator - empty it:
#         for i in iterators:
#             pass
#         return

#     if len(iterators) == 0:
#         return

#     while True:
#         try:
#             results = [iter.next() for iter in iterators]
#         except StopIteration:
#             return results



from __future__ import print_function
#__docformat__ = "restructuredtext en"
# ******NOTICE***************
# optimize.py module by Travis E. Oliphant
#
# You may copy and use this module as you see fit with no
# guarantee implied provided you keep this notice in all copies.
# *****END NOTICE************

# A collection of optimization algorithms.  Version 0.5
# CHANGES
#  Added fminbound (July 2001)
#  Added brute (Aug. 2002)
#  Finished line search satisfying strong Wolfe conditions (Mar. 2004)
#  Updated strong Wolfe conditions line search to use cubic-interpolation (Mar. 2004)

# Small changes applied to code by: Rasmus E. Christiansen

# Minimization routines

__all__ = ['fmin', 'fmin_powell','fmin_bfgs', 'fmin_ncg', 'fmin_cg',
           'fminbound','brent', 'golden','bracket','rosen','rosen_der',
           'rosen_hess', 'rosen_hess_prod', 'brute', 'approx_fprime',
           'line_search', 'check_grad']

__docformat__ = "restructuredtext en"

import numpy
from numpy import atleast_1d, eye, mgrid, argmin, zeros, shape, empty, \
     squeeze, vectorize, asarray, absolute, sqrt, Inf, asfarray, isinf
#from scipy.optimize import linesearch

# These have been copied from Numeric's MLab.py
# I don't think they made the transition to scipy_core
def max(m,axis=0):
    """max(m,axis=0) returns the maximum of m along dimension axis.
    """
    m = asarray(m)
    return numpy.maximum.reduce(m,axis)

def min(m,axis=0):
    """min(m,axis=0) returns the minimum of m along dimension axis.
    """
    m = asarray(m)
    return numpy.minimum.reduce(m,axis)

abs = absolute
import __builtin__
pymin = __builtin__.min
pymax = __builtin__.max
__version__="0.7"
_epsilon = sqrt(numpy.finfo(float).eps)

def vecnorm(x, ord=2):
    if ord == Inf:
        return numpy.amax(abs(x))
    elif ord == -Inf:
        return numpy.amin(abs(x))
    else:
        return numpy.sum(abs(x)**ord,axis=0)**(1.0/ord)

def rosen(x):  # The Rosenbrock function
    x = asarray(x)
    return numpy.sum(100.0*(x[1:]-x[:-1]**2.0)**2.0 + (1-x[:-1])**2.0,axis=0)

def rosen_der(x):
    x = asarray(x)
    xm = x[1:-1]
    xm_m1 = x[:-2]
    xm_p1 = x[2:]
    der = numpy.zeros_like(x)
    der[1:-1] = 200*(xm-xm_m1**2) - 400*(xm_p1 - xm**2)*xm - 2*(1-xm)
    der[0] = -400*x[0]*(x[1]-x[0]**2) - 2*(1-x[0])
    der[-1] = 200*(x[-1]-x[-2]**2)
    return der

def rosen_hess(x):
    x = atleast_1d(x)
    H = numpy.diag(-400*x[:-1],1) - numpy.diag(400*x[:-1],-1)
    diagonal = numpy.zeros(len(x), dtype=x.dtype)
    diagonal[0] = 1200*x[0]-400*x[1]+2
    diagonal[-1] = 200
    diagonal[1:-1] = 202 + 1200*x[1:-1]**2 - 400*x[2:]
    H = H + numpy.diag(diagonal)
    return H

def rosen_hess_prod(x,p):
    x = atleast_1d(x)
    Hp = numpy.zeros(len(x), dtype=x.dtype)
    Hp[0] = (1200*x[0]**2 - 400*x[1] + 2)*p[0] - 400*x[0]*p[1]
    Hp[1:-1] = -400*x[:-2]*p[:-2]+(202+1200*x[1:-1]**2-400*x[2:])*p[1:-1] \
               -400*x[1:-1]*p[2:]
    Hp[-1] = -400*x[-2]*p[-2] + 200*p[-1]
    return Hp

def wrap_function(function, args):
    ncalls = [0]
    def function_wrapper(x):
        ncalls[0] += 1
        return function(x, *args)
    return ncalls, function_wrapper

def fmin(func, x0, args=(), xtol=1e-4, ftol=1e-4, delta=0.05, maxiter=None, maxfun=None,
         full_output=0, disp=1, retall=0, callback=None):
    """Minimize a function using the downhill simplex algorithm.

    :Parameters:

      func : callable func(x,*args)
          The objective function to be minimized.
      x0 : ndarray
          Initial guess.
      args : tuple
          Extra arguments passed to func, i.e. ``f(x,*args)``.
      callback : callable
          Called after each iteration, as callback(xk), where xk is the
          current parameter vector.

    :Returns: (xopt, {fopt, iter, funcalls, warnflag})

      xopt : ndarray
          Parameter that minimizes function.
      fopt : float
          Value of function at minimum: ``fopt = func(xopt)``.
      iter : int
          Number of iterations performed.
      funcalls : int
          Number of function calls made.
      warnflag : int
          1 : Maximum number of function evaluations made.
          2 : Maximum number of iterations reached.
      allvecs : list
          Solution at each iteration.

    *Other Parameters*:

      xtol : float
          Relative error in xopt acceptable for convergence.
      ftol : number
          Relative error in func(xopt) acceptable for convergence.
      delta : float
          Stepsize for the initial simplex. Default is 0.05.
      maxiter : int
          Maximum number of iterations to perform.
      maxfun : number
          Maximum number of function evaluations to make.
      full_output : bool
          Set to True if fval and warnflag outputs are desired.
      disp : bool
          Set to True to print convergence messages.
      retall : bool
          Set to True to return list of solutions at each iteration.

    :Notes:

        Uses a Nelder-Mead simplex algorithm to find the minimum of
        function of one or more variables.

    """
    fcalls, func = wrap_function(func, args)
    x0 = asfarray(x0).flatten()
    N = len(x0)
    rank = len(x0.shape)
    if not -1 < rank < 2:
        raise ValueError("Initial guess must be a scalar or rank-1 sequence.")
    if maxiter is None:
        maxiter = N * 200
    if maxfun is None:
        maxfun = N * 200

    rho = 1; chi = 2; psi = 0.5; sigma = 0.5;
    one2np1 = range(1,N+1)

    if rank == 0:
        sim = numpy.zeros((N+1,), dtype=x0.dtype)
    else:
        sim = numpy.zeros((N+1,N), dtype=x0.dtype)
    fsim = numpy.zeros((N+1,), float)
    sim[0] = x0
    if retall:
        allvecs = [sim[0]]
    fsim[0] = func(x0)
    # the stepsize has been changed from the normal 5% to only 1% of the parameter
    # value
    nonzdelt = delta #0.01
    zdelt = 0.00025
    for k in range(0,N):
        y = numpy.array(x0,copy=True)
        if y[k] != 0:
            y[k] = (1+nonzdelt)*y[k]
        else:
            y[k] = zdelt

        sim[k+1] = y
        f = func(y)
        fsim[k+1] = f

    ind = numpy.argsort(fsim)
    fsim = numpy.take(fsim,ind,0)
    # sort so sim[0,:] has the lowest function value
    sim = numpy.take(sim,ind,0)

    iterations = 1

    while (fcalls[0] < maxfun and iterations < maxiter):
        if (max(numpy.ravel(abs(sim[1:]-sim[0]))) <= xtol \
            and max(abs(fsim[0]-fsim[1:])) <= ftol):
            break

        xbar = numpy.add.reduce(sim[:-1],0) / N
        xr = (1+rho)*xbar - rho*sim[-1]
        fxr = func(xr)
        doshrink = 0

        if fxr < fsim[0]:
            xe = (1+rho*chi)*xbar - rho*chi*sim[-1]
            fxe = func(xe)

            if fxe < fxr:
                sim[-1] = xe
                fsim[-1] = fxe
            else:
                sim[-1] = xr
                fsim[-1] = fxr
        else: # fsim[0] <= fxr
            if fxr < fsim[-2]:
                sim[-1] = xr
                fsim[-1] = fxr
            else: # fxr >= fsim[-2]
                # Perform contraction
                if fxr < fsim[-1]:
                    xc = (1+psi*rho)*xbar - psi*rho*sim[-1]
                    fxc = func(xc)

                    if fxc <= fxr:
                        sim[-1] = xc
                        fsim[-1] = fxc
                    else:
                        doshrink=1
                else:
                    # Perform an inside contraction
                    xcc = (1-psi)*xbar + psi*sim[-1]
                    fxcc = func(xcc)

                    if fxcc < fsim[-1]:
                        sim[-1] = xcc
                        fsim[-1] = fxcc
                    else:
                        doshrink = 1

                if doshrink:
                    for j in one2np1:
                        sim[j] = sim[0] + sigma*(sim[j] - sim[0])
                        fsim[j] = func(sim[j])

        ind = numpy.argsort(fsim)
        sim = numpy.take(sim,ind,0)
        fsim = numpy.take(fsim,ind,0)
        if callback is not None:
            callback(sim[0])
        iterations += 1
        if retall:
            allvecs.append(sim[0])

    x = sim[0]
    fval = min(fsim)
    warnflag = 0

    if fcalls[0] >= maxfun:
        warnflag = 1
        if disp:
            print("Warning: Maximum number of function evaluations has "\
                  "been exceeded.")
    elif iterations >= maxiter:
        warnflag = 2
        if disp:
            print("Warning: Maximum number of iterations has been exceeded")
    else:
        if disp:
            print("Optimization terminated successfully.")
            print("         Current function value: %f" % fval)
            print("         Iterations: %d" % iterations)
            print("         Function evaluations: %d" % fcalls[0])


    if full_output:
        retlist = x, fval, iterations, fcalls[0], warnflag
        if retall:
            retlist += (allvecs,)
    else:
        retlist = x
        if retall:
            retlist = (x, allvecs)

    return retlist



def _cubicmin(a,fa,fpa,b,fb,c,fc):
    # finds the minimizer for a cubic polynomial that goes through the
    #  points (a,fa), (b,fb), and (c,fc) with derivative at a of fpa.
    #
    # if no minimizer can be found return None
    #
    # f(x) = A *(x-a)^3 + B*(x-a)^2 + C*(x-a) + D

    C = fpa
    D = fa
    db = b-a
    dc = c-a
    if (db == 0) or (dc == 0) or (b==c): return None
    denom = (db*dc)**2 * (db-dc)
    d1 = empty((2,2))
    d1[0,0] = dc**2
    d1[0,1] = -db**2
    d1[1,0] = -dc**3
    d1[1,1] = db**3
    [A,B] = numpy.dot(d1,asarray([fb-fa-C*db,fc-fa-C*dc]).flatten())
    A /= denom
    B /= denom
    radical = B*B-3*A*C
    if radical < 0:  return None
    if (A == 0): return None
    xmin = a + (-B + sqrt(radical))/(3*A)
    return xmin


def _quadmin(a,fa,fpa,b,fb):
    # finds the minimizer for a quadratic polynomial that goes through
    #  the points (a,fa), (b,fb) with derivative at a of fpa
    # f(x) = B*(x-a)^2 + C*(x-a) + D
    D = fa
    C = fpa
    db = b-a*1.0
    if (db==0): return None
    B = (fb-D-C*db)/(db*db)
    if (B <= 0): return None
    xmin = a  - C / (2.0*B)
    return xmin

def zoom(a_lo, a_hi, phi_lo, phi_hi, derphi_lo,
         phi, derphi, phi0, derphi0, c1, c2):
    maxiter = 10
    i = 0
    delta1 = 0.2  # cubic interpolant check
    delta2 = 0.1  # quadratic interpolant check
    phi_rec = phi0
    a_rec = 0
    while 1:
        # interpolate to find a trial step length between a_lo and a_hi
        # Need to choose interpolation here.  Use cubic interpolation and then if the
        #  result is within delta * dalpha or outside of the interval bounded by a_lo or a_hi
        #  then use quadratic interpolation, if the result is still too close, then use bisection

        dalpha = a_hi-a_lo;
        if dalpha < 0: a,b = a_hi,a_lo
        else: a,b = a_lo, a_hi

        # minimizer of cubic interpolant
        #    (uses phi_lo, derphi_lo, phi_hi, and the most recent value of phi)
        #      if the result is too close to the end points (or out of the interval)
        #         then use quadratic interpolation with phi_lo, derphi_lo and phi_hi
        #      if the result is stil too close to the end points (or out of the interval)
        #         then use bisection

        if (i > 0):
            cchk = delta1*dalpha
            a_j = _cubicmin(a_lo, phi_lo, derphi_lo, a_hi, phi_hi, a_rec, phi_rec)
        if (i==0) or (a_j is None) or (a_j > b-cchk) or (a_j < a+cchk):
            qchk = delta2*dalpha
            a_j = _quadmin(a_lo, phi_lo, derphi_lo, a_hi, phi_hi)
            if (a_j is None) or (a_j > b-qchk) or (a_j < a+qchk):
                a_j = a_lo + 0.5*dalpha
#                print "Using bisection."
#            else: print "Using quadratic."
#        else: print "Using cubic."

        # Check new value of a_j

        phi_aj = phi(a_j)
        if (phi_aj > phi0 + c1*a_j*derphi0) or (phi_aj >= phi_lo):
            phi_rec = phi_hi
            a_rec = a_hi
            a_hi = a_j
            phi_hi = phi_aj
        else:
            derphi_aj = derphi(a_j)
            if abs(derphi_aj) <= -c2*derphi0:
                a_star = a_j
                val_star = phi_aj
                valprime_star = derphi_aj
                break
            if derphi_aj*(a_hi - a_lo) >= 0:
                phi_rec = phi_hi
                a_rec = a_hi
                a_hi = a_lo
                phi_hi = phi_lo
            else:
                phi_rec = phi_lo
                a_rec = a_lo
            a_lo = a_j
            phi_lo = phi_aj
            derphi_lo = derphi_aj
        i += 1
        if (i > maxiter):
            a_star = a_j
            val_star = phi_aj
            valprime_star = None
            break
    return a_star, val_star, valprime_star

def line_search(f, myfprime, xk, pk, gfk, old_fval, old_old_fval,
                args=(), c1=1e-4, c2=0.9, amax=50):
    """Find alpha that satisfies strong Wolfe conditions.

    :Parameters:

        f : callable f(x,*args)
            Objective function.
        myfprime : callable f'(x,*args)
            Objective function gradient (can be None).
        xk : ndarray
            Starting point.
        pk : ndarray
            Search direction.
        gfk : ndarray
            Gradient value for x=xk (xk being the current parameter
            estimate).
        args : tuple
            Additional arguments passed to objective function.
        c1 : float
            Parameter for Armijo condition rule.
        c2 : float
            Parameter for curvature condition rule.

    :Returns:

        alpha0 : float
            Alpha for which ``x_new = x0 + alpha * pk``.
        fc : int
            Number of function evaluations made.
        gc : int
            Number of gradient evaluations made.

    :Notes:

        Uses the line search algorithm to enforce strong Wolfe
        conditions.  See Wright and Nocedal, 'Numerical Optimization',
        1999, pg. 59-60.

        For the zoom phase it uses an algorithm by [...].

    """

    global _ls_fc, _ls_gc, _ls_ingfk
    _ls_fc = 0
    _ls_gc = 0
    _ls_ingfk = None
    def phi(alpha):
        global _ls_fc
        _ls_fc += 1
        return f(xk+alpha*pk,*args)

    if isinstance(myfprime,type(())):
        def phiprime(alpha):
            global _ls_fc, _ls_ingfk
            _ls_fc += len(xk)+1
            eps = myfprime[1]
            fprime = myfprime[0]
            newargs = (f,eps) + args
            _ls_ingfk = fprime(xk+alpha*pk,*newargs)  # store for later use
            return numpy.dot(_ls_ingfk,pk)
    else:
        fprime = myfprime
        def phiprime(alpha):
            global _ls_gc, _ls_ingfk
            _ls_gc += 1
            _ls_ingfk = fprime(xk+alpha*pk,*args)  # store for later use
            return numpy.dot(_ls_ingfk,pk)

    alpha0 = 0
    phi0 = old_fval
    derphi0 = numpy.dot(gfk,pk)

    alpha1 = pymin(1.0,1.01*2*(phi0-old_old_fval)/derphi0)

    if alpha1 == 0:
        # This shouldn't happen. Perhaps the increment has slipped below
        # machine precision?  For now, set the return variables skip the
        # useless while loop, and raise warnflag=2 due to possible imprecision.
        alpha_star = None
        fval_star = old_fval
        old_fval = old_old_fval
        fprime_star = None

    phi_a1 = phi(alpha1)
    #derphi_a1 = phiprime(alpha1)  evaluated below

    phi_a0 = phi0
    derphi_a0 = derphi0

    i = 1
    maxiter = 10
    while 1:         # bracketing phase
        if alpha1 == 0:
            break
        if (phi_a1 > phi0 + c1*alpha1*derphi0) or \
           ((phi_a1 >= phi_a0) and (i > 1)):
            alpha_star, fval_star, fprime_star = \
                        zoom(alpha0, alpha1, phi_a0,
                             phi_a1, derphi_a0, phi, phiprime,
                             phi0, derphi0, c1, c2)
            break

        derphi_a1 = phiprime(alpha1)
        if (abs(derphi_a1) <= -c2*derphi0):
            alpha_star = alpha1
            fval_star = phi_a1
            fprime_star = derphi_a1
            break

        if (derphi_a1 >= 0):
            alpha_star, fval_star, fprime_star = \
                        zoom(alpha1, alpha0, phi_a1,
                             phi_a0, derphi_a1, phi, phiprime,
                             phi0, derphi0, c1, c2)
            break

        alpha2 = 2 * alpha1   # increase by factor of two on each iteration
        i = i + 1
        alpha0 = alpha1
        alpha1 = alpha2
        phi_a0 = phi_a1
        phi_a1 = phi(alpha1)
        derphi_a0 = derphi_a1

        # stopping test if lower function not found
        if (i > maxiter):
            alpha_star = alpha1
            fval_star = phi_a1
            fprime_star = None
            break

    if fprime_star is not None:
        # fprime_star is a number (derphi) -- so use the most recently
        # calculated gradient used in computing it derphi = gfk*pk
        # this is the gradient at the next step no need to compute it
        # again in the outer loop.
        fprime_star = _ls_ingfk

    return alpha_star, _ls_fc, _ls_gc, fval_star, old_fval, fprime_star


def line_search_BFGS(f, xk, pk, gfk, old_fval, args=(), c1=1e-4, alpha0=1):
    """Minimize over alpha, the function ``f(xk+alpha pk)``.

    Uses the interpolation algorithm (Armiijo backtracking) as suggested by
    Wright and Nocedal in 'Numerical Optimization', 1999, pg. 56-57

    :Returns: (alpha, fc, gc)

    """

    xk = atleast_1d(xk)
    fc = 0
    phi0 = old_fval # compute f(xk) -- done in past loop
    phi_a0 = f(*((xk+alpha0*pk,)+args))
    fc = fc + 1
    derphi0 = numpy.dot(gfk,pk)

    if (phi_a0 <= phi0 + c1*alpha0*derphi0):
        return alpha0, fc, 0, phi_a0

    # Otherwise compute the minimizer of a quadratic interpolant:

    alpha1 = -(derphi0) * alpha0**2 / 2.0 / (phi_a0 - phi0 - derphi0 * alpha0)
    phi_a1 = f(*((xk+alpha1*pk,)+args))
    fc = fc + 1

    if (phi_a1 <= phi0 + c1*alpha1*derphi0):
        return alpha1, fc, 0, phi_a1

    # Otherwise loop with cubic interpolation until we find an alpha which
    # satifies the first Wolfe condition (since we are backtracking, we will
    # assume that the value of alpha is not too small and satisfies the second
    # condition.

    while 1:       # we are assuming pk is a descent direction
        factor = alpha0**2 * alpha1**2 * (alpha1-alpha0)
        a = alpha0**2 * (phi_a1 - phi0 - derphi0*alpha1) - \
            alpha1**2 * (phi_a0 - phi0 - derphi0*alpha0)
        a = a / factor
        b = -alpha0**3 * (phi_a1 - phi0 - derphi0*alpha1) + \
            alpha1**3 * (phi_a0 - phi0 - derphi0*alpha0)
        b = b / factor

        alpha2 = (-b + numpy.sqrt(abs(b**2 - 3 * a * derphi0))) / (3.0*a)
        phi_a2 = f(*((xk+alpha2*pk,)+args))
        fc = fc + 1

        if (phi_a2 <= phi0 + c1*alpha2*derphi0):
            return alpha2, fc, 0, phi_a2

        if (alpha1 - alpha2) > alpha1 / 2.0 or (1 - alpha2/alpha1) < 0.96:
            alpha2 = alpha1 / 2.0

        alpha0 = alpha1
        alpha1 = alpha2
        phi_a0 = phi_a1
        phi_a1 = phi_a2


def approx_fprime(xk,f,epsilon,*args):
    f0 = f(*((xk,)+args))
    grad = numpy.zeros((len(xk),), float)
    ei = numpy.zeros((len(xk),), float)
    for k in range(len(xk)):
        ei[k] = epsilon
        grad[k] = (f(*((xk+ei,)+args)) - f0)/epsilon
        ei[k] = 0.0
    return grad

def check_grad(func, grad, x0, *args):
    return sqrt(sum((grad(x0,*args)-approx_fprime(x0,func,_epsilon,*args))**2))

def approx_fhess_p(x0,p,fprime,epsilon,*args):
    f2 = fprime(*((x0+epsilon*p,)+args))
    f1 = fprime(*((x0,)+args))
    return (f2 - f1)/epsilon


def fmin_bfgs(f, x0, fprime=None, args=(), gtol=1e-5, norm=Inf,
              epsilon=_epsilon, maxiter=None, full_output=0, disp=1,
              retall=0, callback=None):
    """Minimize a function using the BFGS algorithm.

    :Parameters:

      f : callable f(x,*args)
          Objective function to be minimized.
      x0 : ndarray
          Initial guess.
      fprime : callable f'(x,*args)
          Gradient of f.
      args : tuple
          Extra arguments passed to f and fprime.
      gtol : float
          Gradient norm must be less than gtol before succesful termination.
      norm : float
          Order of norm (Inf is max, -Inf is min)
      epsilon : int or ndarray
          If fprime is approximated, use this value for the step size.
      callback : callable
          An optional user-supplied function to call after each
          iteration.  Called as callback(xk), where xk is the
          current parameter vector.

    :Returns: (xopt, {fopt, gopt, Hopt, func_calls, grad_calls, warnflag}, <allvecs>)

        xopt : ndarray
            Parameters which minimize f, i.e. f(xopt) == fopt.
        fopt : float
            Minimum value.
        gopt : ndarray
            Value of gradient at minimum, f'(xopt), which should be near 0.
        Bopt : ndarray
            Value of 1/f''(xopt), i.e. the inverse hessian matrix.
        func_calls : int
            Number of function_calls made.
        grad_calls : int
            Number of gradient calls made.
        warnflag : integer
            1 : Maximum number of iterations exceeded.
            2 : Gradient and/or function calls not changing.
        allvecs  :  list
            Results at each iteration.  Only returned if retall is True.

    *Other Parameters*:
        maxiter : int
            Maximum number of iterations to perform.
        full_output : bool
            If True,return fopt, func_calls, grad_calls, and warnflag
            in addition to xopt.
        disp : bool
            Print convergence message if True.
        retall : bool
            Return a list of results at each iteration if True.

    :Notes:

        Optimize the function, f, whose gradient is given by fprime
        using the quasi-Newton method of Broyden, Fletcher, Goldfarb,
        and Shanno (BFGS) See Wright, and Nocedal 'Numerical
        Optimization', 1999, pg. 198.

    *See Also*:

      scikits.openopt : SciKit which offers a unified syntax to call
                        this and other solvers.

    """
    x0 = asarray(x0).squeeze()
    if x0.ndim == 0:
        x0.shape = (1,)
    if maxiter is None:
        maxiter = len(x0)*200
    func_calls, f = wrap_function(f, args)
    if fprime is None:
        grad_calls, myfprime = wrap_function(approx_fprime, (f, epsilon))
    else:
        grad_calls, myfprime = wrap_function(fprime, args)
    gfk = myfprime(x0)
    k = 0
    N = len(x0)
    I = numpy.eye(N,dtype=int)
    Hk = I
    old_fval = f(x0)
    old_old_fval = old_fval + 5000
    xk = x0
    if retall:
        allvecs = [x0]
    sk = [2*gtol]
    warnflag = 0
    gnorm = vecnorm(gfk,ord=norm)
    while (gnorm > gtol) and (k < maxiter):
        pk = -numpy.dot(Hk,gfk)
        alpha_k, fc, gc, old_fval, old_old_fval, gfkp1 = \
           linesearch.line_search(f,myfprime,xk,pk,gfk,
                                  old_fval,old_old_fval)
        if alpha_k is None:  # line search failed try different one.
            alpha_k, fc, gc, old_fval, old_old_fval, gfkp1 = \
                     line_search(f,myfprime,xk,pk,gfk,
                                 old_fval,old_old_fval)
            if alpha_k is None:
                # This line search also failed to find a better solution.
                warnflag = 2
                break
        xkp1 = xk + alpha_k * pk
        if retall:
            allvecs.append(xkp1)
        sk = xkp1 - xk
        xk = xkp1
        if gfkp1 is None:
            gfkp1 = myfprime(xkp1)

        yk = gfkp1 - gfk
        gfk = gfkp1
        if callback is not None:
            callback(xk)
        k += 1
        gnorm = vecnorm(gfk,ord=norm)
        if (gnorm <= gtol):
            break

        try: # this was handled in numeric, let it remaines for more safety
            rhok = 1.0 / (numpy.dot(yk,sk))
        except ZeroDivisionError:
            rhok = 1000.0
            print("Divide-by-zero encountered: rhok assumed large")
        if isinf(rhok): # this is patch for numpy
            rhok = 1000.0
            print("Divide-by-zero encountered: rhok assumed large")
        A1 = I - sk[:,numpy.newaxis] * yk[numpy.newaxis,:] * rhok
        A2 = I - yk[:,numpy.newaxis] * sk[numpy.newaxis,:] * rhok
        Hk = numpy.dot(A1,numpy.dot(Hk,A2)) + rhok * sk[:,numpy.newaxis] \
                 * sk[numpy.newaxis,:]

    if disp or full_output:
        fval = old_fval
    if warnflag == 2:
        if disp:
            print("Warning: Desired error not necessarily achieved" \
                  "due to precision loss")
            print("         Current function value: %f" % fval)
            print("         Iterations: %d" % k)
            print("         Function evaluations: %d" % func_calls[0])
            print("         Gradient evaluations: %d" % grad_calls[0])

    elif k >= maxiter:
        warnflag = 1
        if disp:
            print("Warning: Maximum number of iterations has been exceeded")
            print("         Current function value: %f" % fval)
            print("         Iterations: %d" % k)
            print("         Function evaluations: %d" % func_calls[0])
            print("         Gradient evaluations: %d" % grad_calls[0])
    else:
        if disp:
            print("Optimization terminated successfully.")
            print("         Current function value: %f" % fval)
            print("         Iterations: %d" % k)
            print("         Function evaluations: %d" % func_calls[0])
            print("         Gradient evaluations: %d" % grad_calls[0])

    if full_output:
        retlist = xk, fval, gfk, Hk, func_calls[0], grad_calls[0], warnflag
        if retall:
            retlist += (allvecs,)
    else:
        retlist = xk
        if retall:
            retlist = (xk, allvecs)

    return retlist


def fmin_cg(f, x0, fprime=None, args=(), gtol=1e-5, norm=Inf, epsilon=_epsilon,
              maxiter=None, full_output=0, disp=1, retall=0, callback=None):
    """Minimize a function using a nonlinear conjugate gradient algorithm.

    :Parameters:
        f : callable f(x,*args)
            Objective function to be minimized.
        x0 : ndarray
            Initial guess.
        fprime : callable f'(x,*args)
            Function which computes the gradient of f.
        args : tuple
            Extra arguments passed to f and fprime.
        gtol : float
            Stop when norm of gradient is less than gtol.
        norm : float
            Order of vector norm to use.  -Inf is min, Inf is max.
        epsilon : float or ndarray
            If fprime is approximated, use this value for the step
            size (can be scalar or vector).
        callback : callable
            An optional user-supplied function, called after each
            iteration.  Called as callback(xk), where xk is the
            current parameter vector.

    :Returns: (xopt, {fopt, func_calls, grad_calls, warnflag}, {allvecs})

        xopt : ndarray
            Parameters which minimize f, i.e. f(xopt) == fopt.
        fopt : float
            Minimum value found, f(xopt).
        func_calls : int
            The number of function_calls made.
        grad_calls : int
            The number of gradient calls made.
        warnflag : int
            1 : Maximum number of iterations exceeded.
            2 : Gradient and/or function calls not changing.
        allvecs : ndarray
            If retall is True (see other parameters below), then this
            vector containing the result at each iteration is returned.

    *Other Parameters*:
      maxiter : int
          Maximum number of iterations to perform.
      full_output : bool
          If True then return fopt, func_calls, grad_calls, and
          warnflag in addition to xopt.
      disp : bool
          Print convergence message if True.
      retall : bool
        return a list of results at each iteration if True.

    :Notes:

        Optimize the function, f, whose gradient is given by fprime
        using the nonlinear conjugate gradient algorithm of Polak and
        Ribiere See Wright, and Nocedal 'Numerical Optimization',
        1999, pg. 120-122.

    """
    x0 = asarray(x0).flatten()
    if maxiter is None:
        maxiter = len(x0)*200
    func_calls, f = wrap_function(f, args)
    if fprime is None:
        grad_calls, myfprime = wrap_function(approx_fprime, (f, epsilon))
    else:
        grad_calls, myfprime = wrap_function(fprime, args)
    gfk = myfprime(x0)
    k = 0
    N = len(x0)
    xk = x0
    old_fval = f(xk)
    old_old_fval = old_fval + 5000

    if retall:
        allvecs = [xk]
    sk = [2*gtol]
    warnflag = 0
    pk = -gfk
    gnorm = vecnorm(gfk,ord=norm)
    while (gnorm > gtol) and (k < maxiter):
        deltak = numpy.dot(gfk,gfk)

        # These values are modified by the line search, even if it fails
        old_fval_backup = old_fval
        old_old_fval_backup = old_old_fval

        alpha_k, fc, gc, old_fval, old_old_fval, gfkp1 = \
           linesearch.line_search(f,myfprime,xk,pk,gfk,old_fval,
                                  old_old_fval,c2=0.4)
        if alpha_k is None:  # line search failed -- use different one.
            alpha_k, fc, gc, old_fval, old_old_fval, gfkp1 = \
                     line_search(f,myfprime,xk,pk,gfk,
                                 old_fval_backup,old_old_fval_backup)
            if alpha_k is None or alpha_k == 0:
                # This line search also failed to find a better solution.
                warnflag = 2
                break
        xk = xk + alpha_k*pk
        if retall:
            allvecs.append(xk)
        if gfkp1 is None:
            gfkp1 = myfprime(xk)
        yk = gfkp1 - gfk
        beta_k = pymax(0,numpy.dot(yk,gfkp1)/deltak)
        pk = -gfkp1 + beta_k * pk
        gfk = gfkp1
        gnorm = vecnorm(gfk,ord=norm)
        if callback is not None:
            callback(xk)
        k += 1


    if disp or full_output:
        fval = old_fval
    if warnflag == 2:
        if disp:
            print("Warning: Desired error not necessarily achieved due to precision loss")
            print("         Current function value: %f" % fval)
            print("         Iterations: %d" % k)
            print("         Function evaluations: %d" % func_calls[0])
            print("         Gradient evaluations: %d" % grad_calls[0])

    elif k >= maxiter:
        warnflag = 1
        if disp:
            print("Warning: Maximum number of iterations has been exceeded")
            print("         Current function value: %f" % fval)
            print("         Iterations: %d" % k)
            print("         Function evaluations: %d" % func_calls[0])
            print("         Gradient evaluations: %d" % grad_calls[0])
    else:
        if disp:
            print("Optimization terminated successfully.")
            print("         Current function value: %f" % fval)
            print("         Iterations: %d" % k)
            print("         Function evaluations: %d" % func_calls[0])
            print("         Gradient evaluations: %d" % grad_calls[0])


    if full_output:
        retlist = xk, fval, func_calls[0], grad_calls[0], warnflag
        if retall:
            retlist += (allvecs,)
    else:
        retlist = xk
        if retall:
            retlist = (xk, allvecs)

    return retlist

def fmin_ncg(f, x0, fprime, fhess_p=None, fhess=None, args=(), avextol=1e-5,
             epsilon=_epsilon, maxiter=None, full_output=0, disp=1, retall=0,
             callback=None):
    """Minimize a function using the Newton-CG method.

    :Parameters:

        f : callable f(x,*args)
            Objective function to be minimized.
        x0 : ndarray
            Initial guess.
        fprime : callable f'(x,*args)
            Gradient of f.
        fhess_p : callable fhess_p(x,p,*args)
            Function which computes the Hessian of f times an
            arbitrary vector, p.
        fhess : callable fhess(x,*args)
            Function to compute the Hessian matrix of f.
        args : tuple
            Extra arguments passed to f, fprime, fhess_p, and fhess
            (the same set of extra arguments is supplied to all of
            these functions).
        epsilon : float or ndarray
            If fhess is approximated, use this value for the step size.
        callback : callable
            An optional user-supplied function which is called after
            each iteration.  Called as callback(xk), where xk is the
            current parameter vector.

    :Returns: (xopt, {fopt, fcalls, gcalls, hcalls, warnflag},{allvecs})

        xopt : ndarray
            Parameters which minimizer f, i.e. ``f(xopt) == fopt``.
        fopt : float
            Value of the function at xopt, i.e. ``fopt = f(xopt)``.
        fcalls : int
            Number of function calls made.
        gcalls : int
            Number of gradient calls made.
        hcalls : int
            Number of hessian calls made.
        warnflag : int
            Warnings generated by the algorithm.
            1 : Maximum number of iterations exceeded.
        allvecs : list
            The result at each iteration, if retall is True (see below).

    *Other Parameters*:

        avextol : float
            Convergence is assumed when the average relative error in
            the minimizer falls below this amount.
        maxiter : int
            Maximum number of iterations to perform.
        full_output : bool
            If True, return the optional outputs.
        disp : bool
            If True, print convergence message.
        retall : bool
            If True, return a list of results at each iteration.

    :Notes:
      1. scikits.openopt offers a unified syntax to call this and other solvers.
      2. Only one of `fhess_p` or `fhess` need to be given.  If `fhess`
      is provided, then `fhess_p` will be ignored.  If neither `fhess`
      nor `fhess_p` is provided, then the hessian product will be
      approximated using finite differences on `fprime`. `fhess_p`
      must compute the hessian times an arbitrary vector. If it is not
      given, finite-differences on `fprime` are used to compute
      it. See Wright, and Nocedal 'Numerical Optimization', 1999,
      pg. 140.

    """
    x0 = asarray(x0).flatten()
    fcalls, f = wrap_function(f, args)
    gcalls, fprime = wrap_function(fprime, args)
    hcalls = 0
    if maxiter is None:
        maxiter = len(x0)*200

    xtol = len(x0)*avextol
    update = [2*xtol]
    xk = x0
    if retall:
        allvecs = [xk]
    k = 0
    old_fval = f(x0)
    while (numpy.add.reduce(abs(update)) > xtol) and (k < maxiter):
        # Compute a search direction pk by applying the CG method to
        #  del2 f(xk) p = - grad f(xk) starting from 0.
        b = -fprime(xk)
        maggrad = numpy.add.reduce(abs(b))
        eta = min([0.5,numpy.sqrt(maggrad)])
        termcond = eta * maggrad
        xsupi = zeros(len(x0), dtype=x0.dtype)
        ri = -b
        psupi = -ri
        i = 0
        dri0 = numpy.dot(ri,ri)

        if fhess is not None:             # you want to compute hessian once.
            A = fhess(*(xk,)+args)
            hcalls = hcalls + 1

        while numpy.add.reduce(abs(ri)) > termcond:
            if fhess is None:
                if fhess_p is None:
                    Ap = approx_fhess_p(xk,psupi,fprime,epsilon)
                else:
                    Ap = fhess_p(xk,psupi, *args)
                    hcalls = hcalls + 1
            else:
                Ap = numpy.dot(A,psupi)
            # check curvature
            Ap = asarray(Ap).squeeze() # get rid of matrices...
            curv = numpy.dot(psupi,Ap)
            if curv == 0.0:
                break
            elif curv < 0:
                if (i > 0):
                    break
                else:
                    xsupi = xsupi + dri0/curv * psupi
                    break
            alphai = dri0 / curv
            xsupi = xsupi + alphai * psupi
            ri = ri + alphai * Ap
            dri1 = numpy.dot(ri,ri)
            betai = dri1 / dri0
            psupi = -ri + betai * psupi
            i = i + 1
            dri0 = dri1          # update numpy.dot(ri,ri) for next time.

        pk = xsupi  # search direction is solution to system.
        gfk = -b    # gradient at xk
        alphak, fc, gc, old_fval = line_search_BFGS(f,xk,pk,gfk,old_fval)

        update = alphak * pk
        xk = xk + update        # upcast if necessary
        if callback is not None:
            callback(xk)
        if retall:
            allvecs.append(xk)
        k += 1

    if disp or full_output:
        fval = old_fval
    if k >= maxiter:
        warnflag = 1
        if disp:
            print("Warning: Maximum number of iterations has been exceeded")
            print("         Current function value: %f" % fval)
            print("         Iterations: %d" % k)
            print("         Function evaluations: %d" % fcalls[0])
            print("         Gradient evaluations: %d" % gcalls[0])
            print("         Hessian evaluations: %d" % hcalls)
    else:
        warnflag = 0
        if disp:
            print("Optimization terminated successfully.")
            print("         Current function value: %f" % fval)
            print("         Iterations: %d" % k)
            print("         Function evaluations: %d" % fcalls[0])
            print("         Gradient evaluations: %d" % gcalls[0])
            print("         Hessian evaluations: %d" % hcalls)

    if full_output:
        retlist = xk, fval, fcalls[0], gcalls[0], hcalls, warnflag
        if retall:
            retlist += (allvecs,)
    else:
        retlist = xk
        if retall:
            retlist = (xk, allvecs)

    return retlist


def fminbound(func, x1, x2, args=(), xtol=1e-5, maxfun=500,
              full_output=0, disp=1):
    """Bounded minimization for scalar functions.

    :Parameters:

      func : callable f(x,*args)
          Objective function to be minimized (must accept and return scalars).
      x1, x2 : float or array scalar
          The optimization bounds.
      args : tuple
          Extra arguments passed to function.
      xtol : float
          The convergence tolerance.
      maxfun : int
          Maximum number of function evaluations allowed.
      full_output : bool
          If True, return optional outputs.
      disp : int
          If non-zero, print messages.
              0 : no message printing.
              1 : non-convergence notification messages only.
              2 : print a message on convergence too.
              3 : print iteration results.


    :Returns: (xopt, {fval, ierr, numfunc})

      xopt : ndarray
          Parameters (over given interval) which minimize the
          objective function.
      fval : number
          The function value at the minimum point.
      ierr : int
          An error flag (0 if converged, 1 if maximum number of
          function calls reached).
      numfunc : int
        The number of function calls made.


    :Notes:

        Finds a local minimizer of the scalar function `func` in the
        interval x1 < xopt < x2 using Brent's method.  (See `brent`
        for auto-bracketing).


    """
    # Test bounds are of correct form
    x1 = atleast_1d(x1)
    x2 = atleast_1d(x2)
    if len(x1) != 1 or len(x2) != 1:
        raise ValueError("Optimisation bounds must be scalars" \
                " or length 1 arrays")
    if x1 > x2:
        raise ValueError("The lower bound exceeds the upper bound.")

    flag = 0
    header = ' Func-count     x          f(x)          Procedure'
    step='       initial'

    sqrt_eps = sqrt(2.2e-16)
    golden_mean = 0.5*(3.0-sqrt(5.0))
    a, b = x1, x2
    fulc = a + golden_mean*(b-a)
    nfc, xf = fulc, fulc
    rat = e = 0.0
    x = xf
    fx = func(x,*args)
    num = 1
    fmin_data = (1, xf, fx)

    ffulc = fnfc = fx
    xm = 0.5*(a+b)
    tol1 = sqrt_eps*abs(xf) + xtol / 3.0
    tol2 = 2.0*tol1

    if disp > 2:
        print (" ")
        print (header)
        print("%5.0f   %12.6g %12.6g %s" % (fmin_data + (step,)))


    while ( abs(xf-xm) > (tol2 - 0.5*(b-a)) ):
        golden = 1
        # Check for parabolic fit
        if abs(e) > tol1:
            golden = 0
            r = (xf-nfc)*(fx-ffulc)
            q = (xf-fulc)*(fx-fnfc)
            p = (xf-fulc)*q - (xf-nfc)*r
            q = 2.0*(q-r)
            if q > 0.0: p = -p
            q = abs(q)
            r = e
            e = rat

            # Check for acceptability of parabola
            if ( (abs(p) < abs(0.5*q*r)) and (p > q*(a-xf)) and \
                 (p < q*(b-xf))):
                rat = (p+0.0) / q;
                x = xf + rat
                step = '       parabolic'

                if ((x-a) < tol2) or ((b-x) < tol2):
                    si = numpy.sign(xm-xf) + ((xm-xf)==0)
                    rat = tol1*si
            else:      # do a golden section step
                golden = 1

        if golden:  # Do a golden-section step
            if xf >= xm:
                e=a-xf
            else:
                e=b-xf
            rat = golden_mean*e
            step = '       golden'

        si = numpy.sign(rat) + (rat == 0)
        x = xf + si*max([abs(rat), tol1])
        fu = func(x,*args)
        num += 1
        fmin_data = (num, x, fu)
        if disp > 2:
            print("%5.0f   %12.6g %12.6g %s" % (fmin_data + (step,)))

        if fu <= fx:
            if x >= xf:
                a = xf
            else:
                b = xf
            fulc, ffulc = nfc, fnfc
            nfc, fnfc = xf, fx
            xf, fx = x, fu
        else:
            if x < xf:
                a = x
            else:
                b = x
            if (fu <= fnfc) or (nfc == xf):
                fulc, ffulc = nfc, fnfc
                nfc, fnfc = x, fu
            elif (fu <= ffulc) or (fulc == xf) or (fulc == nfc):
                fulc, ffulc = x, fu

        xm = 0.5*(a+b)
        tol1 = sqrt_eps*abs(xf) + xtol/3.0
        tol2 = 2.0*tol1

        if num >= maxfun:
            flag = 1
            fval = fx
            if disp > 0:
                _endprint(x, flag, fval, maxfun, xtol, disp)
            if full_output:
                return xf, fval, flag, num
            else:
                return xf

    fval = fx
    if disp > 0:
        _endprint(x, flag, fval, maxfun, xtol, disp)

    if full_output:
        return xf, fval, flag, num
    else:
        return xf

class Brent:
    #need to rethink design of __init__
    def __init__(self, func, args=(), tol=1.48e-8, maxiter=500,
                 full_output=0):
        self.func = func
        self.args = args
        self.tol = tol
        self.maxiter = maxiter
        self._mintol = 1.0e-11
        self._cg = 0.3819660
        self.xmin = None
        self.fval = None
        self.iter = 0
        self.funcalls = 0

    #need to rethink design of set_bracket (new options, etc)
    def set_bracket(self, brack = None):
        self.brack = brack
    def get_bracket_info(self):
        #set up
        func = self.func
        args = self.args
        brack = self.brack
        ### BEGIN core bracket_info code ###
        ### carefully DOCUMENT any CHANGES in core ##
        if brack is None:
            xa,xb,xc,fa,fb,fc,funcalls = bracket(func, args=args)
        elif len(brack) == 2:
            xa,xb,xc,fa,fb,fc,funcalls = bracket(func, xa=brack[0], xb=brack[1], args=args)
        elif len(brack) == 3:
            xa,xb,xc = brack
            if (xa > xc):  # swap so xa < xc can be assumed
                dum = xa; xa=xc; xc=dum
            assert ((xa < xb) and (xb < xc)), "Not a bracketing interval."
            fa = func(*((xa,)+args))
            fb = func(*((xb,)+args))
            fc = func(*((xc,)+args))
            assert ((fb<fa) and (fb < fc)), "Not a bracketing interval."
            funcalls = 3
        else:
            raise ValueError("Bracketing interval must be length 2 or 3 sequence.")
        ### END core bracket_info code ###

        return xa,xb,xc,fa,fb,fc,funcalls

    def optimize(self):
        #set up for optimization
        func = self.func
        xa,xb,xc,fa,fb,fc,funcalls = self.get_bracket_info()
        _mintol = self._mintol
        _cg = self._cg
        #################################
        #BEGIN CORE ALGORITHM
        #we are making NO CHANGES in this
        #################################
        x=w=v=xb
        fw=fv=fx=func(*((x,)+self.args))
        if (xa < xc):
            a = xa; b = xc
        else:
            a = xc; b = xa
        deltax= 0.0
        funcalls = 1
        iter = 0
        while (iter < self.maxiter):
            tol1 = self.tol*abs(x) + _mintol
            tol2 = 2.0*tol1
            xmid = 0.5*(a+b)
            if abs(x-xmid) < (tol2-0.5*(b-a)):  # check for convergence
                xmin=x; fval=fx
                break
            if (abs(deltax) <= tol1):
                if (x>=xmid): deltax=a-x       # do a golden section step
                else: deltax=b-x
                rat = _cg*deltax
            else:                              # do a parabolic step
                tmp1 = (x-w)*(fx-fv)
                tmp2 = (x-v)*(fx-fw)
                p = (x-v)*tmp2 - (x-w)*tmp1;
                tmp2 = 2.0*(tmp2-tmp1)
                if (tmp2 > 0.0): p = -p
                tmp2 = abs(tmp2)
                dx_temp = deltax
                deltax= rat
                # check parabolic fit
                if ((p > tmp2*(a-x)) and (p < tmp2*(b-x)) and (abs(p) < abs(0.5*tmp2*dx_temp))):
                    rat = p*1.0/tmp2        # if parabolic step is useful.
                    u = x + rat
                    if ((u-a) < tol2 or (b-u) < tol2):
                        if xmid-x >= 0: rat = tol1
                        else: rat = -tol1
                else:
                    if (x>=xmid): deltax=a-x # if it's not do a golden section step
                    else: deltax=b-x
                    rat = _cg*deltax

            if (abs(rat) < tol1):            # update by at least tol1
                if rat >= 0: u = x + tol1
                else: u = x - tol1
            else:
                u = x + rat
            fu = func(*((u,)+self.args))      # calculate new output value
            funcalls += 1

            if (fu > fx):                 # if it's bigger than current
                if (u<x): a=u
                else: b=u
                if (fu<=fw) or (w==x):
                    v=w; w=u; fv=fw; fw=fu
                elif (fu<=fv) or (v==x) or (v==w):
                    v=u; fv=fu
            else:
                if (u >= x): a = x
                else: b = x
                v=w; w=x; x=u
                fv=fw; fw=fx; fx=fu

            iter += 1
        #################################
        #END CORE ALGORITHM
        #################################

        self.xmin = x
        self.fval = fx
        self.iter = iter
        self.funcalls = funcalls

    def get_result(self, full_output=False):
        if full_output:
            return self.xmin, self.fval, self.iter, self.funcalls
        else:
            return self.xmin


def brent(func, args=(), brack=None, tol=1.48e-8, full_output=0, maxiter=500):
    """Given a function of one-variable and a possible bracketing interval,
    return the minimum of the function isolated to a fractional precision of
    tol.

    :Parameters:

        func : callable f(x,*args)
            Objective function.
        args
            Additional arguments (if present).
        brack : tuple
            Triple (a,b,c) where (a<b<c) and func(b) <
            func(a),func(c).  If bracket consists of two numbers (a,c)
            then they are assumed to be a starting interval for a
            downhill bracket search (see `bracket`); it doesn't always
            mean that the obtained solution will satisfy a<=x<=c.
        full_output : bool
            If True, return all output args (xmin, fval, iter,
            funcalls).

    :Returns:

        xmin : ndarray
            Optimum point.
        fval : float
            Optimum value.
        iter : int
            Number of iterations.
        funcalls : int
            Number of objective function evaluations made.

    Notes
    -----

    Uses inverse parabolic interpolation when possible to speed up convergence
    of golden section method.

    """

    brent = Brent(func=func, args=args, tol=tol, full_output = full_output, maxiter=maxiter)
    brent.set_bracket(brack)
    brent.optimize()
    return brent.get_result(full_output=full_output)



def golden(func, args=(), brack=None, tol=_epsilon, full_output=0):
    """ Given a function of one-variable and a possible bracketing interval,
    return the minimum of the function isolated to a fractional precision of
    tol.

    :Parameters:

        func : callable func(x,*args)
            Objective function to minimize.
        args : tuple
            Additional arguments (if present), passed to func.
        brack : tuple
            Triple (a,b,c), where (a<b<c) and func(b) <
            func(a),func(c).  If bracket consists of two numbers (a,
            c), then they are assumed to be a starting interval for a
            downhill bracket search (see `bracket`); it doesn't always
            mean that obtained solution will satisfy a<=x<=c.
        tol : float
            x tolerance stop criterion
        full_output : bool
            If True, return optional outputs.

    :Notes:

        Uses analog of bisection method to decrease the bracketed
        interval.

    """
    if brack is None:
        xa,xb,xc,fa,fb,fc,funcalls = bracket(func, args=args)
    elif len(brack) == 2:
        xa,xb,xc,fa,fb,fc,funcalls = bracket(func, xa=brack[0], xb=brack[1], args=args)
    elif len(brack) == 3:
        xa,xb,xc = brack
        if (xa > xc):  # swap so xa < xc can be assumed
            dum = xa; xa=xc; xc=dum
        assert ((xa < xb) and (xb < xc)), "Not a bracketing interval."
        fa = func(*((xa,)+args))
        fb = func(*((xb,)+args))
        fc = func(*((xc,)+args))
        assert ((fb<fa) and (fb < fc)), "Not a bracketing interval."
        funcalls = 3
    else:
        raise ValueError("Bracketing interval must be length 2 or 3 sequence.")

    _gR = 0.61803399
    _gC = 1.0-_gR
    x3 = xc
    x0 = xa
    if (abs(xc-xb) > abs(xb-xa)):
        x1 = xb
        x2 = xb + _gC*(xc-xb)
    else:
        x2 = xb
        x1 = xb - _gC*(xb-xa)
    f1 = func(*((x1,)+args))
    f2 = func(*((x2,)+args))
    funcalls += 2
    while (abs(x3-x0) > tol*(abs(x1)+abs(x2))):
        if (f2 < f1):
            x0 = x1; x1 = x2; x2 = _gR*x1 + _gC*x3
            f1 = f2; f2 = func(*((x2,)+args))
        else:
            x3 = x2; x2 = x1; x1 = _gR*x2 + _gC*x0
            f2 = f1; f1 = func(*((x1,)+args))
        funcalls += 1
    if (f1 < f2):
        xmin = x1
        fval = f1
    else:
        xmin = x2
        fval = f2
    if full_output:
        return xmin, fval, funcalls
    else:
        return xmin


def bracket(func, xa=0.0, xb=1.0, args=(), grow_limit=110.0, maxiter=1000):
    """Given a function and distinct initial points, search in the
    downhill direction (as defined by the initital points) and return
    new points xa, xb, xc that bracket the minimum of the function
    f(xa) > f(xb) < f(xc). It doesn't always mean that obtained
    solution will satisfy xa<=x<=xb

    :Parameters:

        func : callable f(x,*args)
            Objective function to minimize.
        xa, xb : float
            Bracketing interval.
        args : tuple
            Additional arguments (if present), passed to `func`.
        grow_limit : float
            Maximum grow limit.
        maxiter : int
            Maximum number of iterations to perform.

    :Returns: xa, xb, xc, fa, fb, fc, funcalls

        xa, xb, xc : float
            Bracket.
        fa, fb, fc : float
            Objective function values in bracket.
        funcalls : int
            Number of function evaluations made.

    """
    _gold = 1.618034
    _verysmall_num = 1e-21
    fa = func(*(xa,)+args)
    fb = func(*(xb,)+args)
    if (fa < fb):                      # Switch so fa > fb
        dum = xa; xa = xb; xb = dum
        dum = fa; fa = fb; fb = dum
    xc = xb + _gold*(xb-xa)
    fc = func(*((xc,)+args))
    funcalls = 3
    iter = 0
    while (fc < fb):
        tmp1 = (xb - xa)*(fb-fc)
        tmp2 = (xb - xc)*(fb-fa)
        val = tmp2-tmp1
        if abs(val) < _verysmall_num:
            denom = 2.0*_verysmall_num
        else:
            denom = 2.0*val
        w = xb - ((xb-xc)*tmp2-(xb-xa)*tmp1)/denom
        wlim = xb + grow_limit*(xc-xb)
        if iter > maxiter:
            raise RuntimeError("Too many iterations.")
        iter += 1
        if (w-xc)*(xb-w) > 0.0:
            fw = func(*((w,)+args))
            funcalls += 1
            if (fw < fc):
                xa = xb; xb=w; fa=fb; fb=fw
                return xa, xb, xc, fa, fb, fc, funcalls
            elif (fw > fb):
                xc = w; fc=fw
                return xa, xb, xc, fa, fb, fc, funcalls
            w = xc + _gold*(xc-xb)
            fw = func(*((w,)+args))
            funcalls += 1
        elif (w-wlim)*(wlim-xc) >= 0.0:
            w = wlim
            fw = func(*((w,)+args))
            funcalls += 1
        elif (w-wlim)*(xc-w) > 0.0:
            fw = func(*((w,)+args))
            funcalls += 1
            if (fw < fc):
                xb=xc; xc=w; w=xc+_gold*(xc-xb)
                fb=fc; fc=fw; fw=func(*((w,)+args))
                funcalls += 1
        else:
            w = xc + _gold*(xc-xb)
            fw = func(*((w,)+args))
            funcalls += 1
        xa=xb; xb=xc; xc=w
        fa=fb; fb=fc; fc=fw
    return xa, xb, xc, fa, fb, fc, funcalls



def _linesearch_powell(func, p, xi, tol=1e-3):
    """Line-search algorithm using fminbound.

    Find the minimium of the function ``func(x0+ alpha*direc)``.

    """
    def myfunc(alpha):
        return func(p + alpha * xi)
    alpha_min, fret, iter, num = brent(myfunc, full_output=1, tol=tol)
    xi = alpha_min*xi
    return squeeze(fret), p+xi, xi


def fmin_powell(func, x0, args=(), xtol=1e-4, ftol=1e-4, maxiter=None,
                maxfun=None, full_output=0, disp=1, retall=0, callback=None,
                direc=None):
    """Minimize a function using modified Powell's method.

    :Parameters:

      func : callable f(x,*args)
          Objective function to be minimized.
      x0 : ndarray
          Initial guess.
      args : tuple
          Eextra arguments passed to func.
      callback : callable
          An optional user-supplied function, called after each
          iteration.  Called as ``callback(xk)``, where ``xk`` is the
          current parameter vector.
      direc : ndarray
          Initial direction set.

    :Returns: (xopt, {fopt, xi, direc, iter, funcalls, warnflag}, {allvecs})

        xopt : ndarray
            Parameter which minimizes `func`.
        fopt : number
            Value of function at minimum: ``fopt = func(xopt)``.
        direc : ndarray
            Current direction set.
        iter : int
            Number of iterations.
        funcalls : int
            Number of function calls made.
        warnflag : int
            Integer warning flag:
                1 : Maximum number of function evaluations.
                2 : Maximum number of iterations.
        allvecs : list
            List of solutions at each iteration.

    *Other Parameters*:

      xtol : float
          Line-search error tolerance.
      ftol : float
          Relative error in ``func(xopt)`` acceptable for convergence.
      maxiter : int
          Maximum number of iterations to perform.
      maxfun : int
          Maximum number of function evaluations to make.
      full_output : bool
          If True, fopt, xi, direc, iter, funcalls, and
          warnflag are returned.
      disp : bool
          If True, print convergence messages.
      retall : bool
          If True, return a list of the solution at each iteration.


    :Notes:

        Uses a modification of Powell's method to find the minimum of
        a function of N variables.

    """
    # we need to use a mutable object here that we can update in the
    # wrapper function
    fcalls, func = wrap_function(func, args)
    x = asarray(x0).flatten()
    if retall:
        allvecs = [x]
    N = len(x)
    rank = len(x.shape)
    if not -1 < rank < 2:
        raise ValueError("Initial guess must be a scalar or rank-1 sequence.")
    if maxiter is None:
        maxiter = N * 1000
    if maxfun is None:
        maxfun = N * 1000


    if direc is None:
        direc = eye(N, dtype=float)
    else:
        direc = asarray(direc, dtype=float)

    fval = squeeze(func(x))
    x1 = x.copy()
    iter = 0;
    ilist = range(N)
    while True:
        fx = fval
        bigind = 0
        delta = 0.0
        for i in ilist:
            direc1 = direc[i]
            fx2 = fval
            fval, x, direc1 = _linesearch_powell(func, x, direc1, tol=xtol*100)
            if (fx2 - fval) > delta:
                delta = fx2 - fval
                bigind = i
        iter += 1
        if callback is not None:
            callback(x)
        if retall:
            allvecs.append(x)
        if (2.0*(fx - fval) <= ftol*(abs(fx)+abs(fval))+1e-20): break
        if fcalls[0] >= maxfun: break
        if iter >= maxiter: break

        # Construct the extrapolated point
        direc1 = x - x1
        x2 = 2*x - x1
        x1 = x.copy()
        fx2 = squeeze(func(x2))

        if (fx > fx2):
            t = 2.0*(fx+fx2-2.0*fval)
            temp = (fx-fval-delta)
            t *= temp*temp
            temp = fx-fx2
            t -= delta*temp*temp
            if t < 0.0:
                fval, x, direc1 = _linesearch_powell(func, x, direc1,
                                                     tol=xtol*100)
                direc[bigind] = direc[-1]
                direc[-1] = direc1

    warnflag = 0
    if fcalls[0] >= maxfun:
        warnflag = 1
        if disp:
            print("Warning: Maximum number of function evaluations has "\
                  "been exceeded.")
    elif iter >= maxiter:
        warnflag = 2
        if disp:
            print("Warning: Maximum number of iterations has been exceeded")
    else:
        if disp:
            print("Optimization terminated successfully.")
            print("         Current function value: %f" % fval)
            print("         Iterations: %d" % iter)
            print("         Function evaluations: %d" % fcalls[0])

    x = squeeze(x)

    if full_output:
        retlist = x, fval, direc, iter, fcalls[0], warnflag
        if retall:
            retlist += (allvecs,)
    else:
        retlist = x
        if retall:
            retlist = (x, allvecs)

    return retlist




def _endprint(x, flag, fval, maxfun, xtol, disp):
    if flag == 0:
        if disp > 1:
            print("\nOptimization terminated successfully;\n" \
                  "The returned value satisfies the termination criteria\n" \
                  "(using xtol = ", xtol, ")")
    if flag == 1:
        print("\nMaximum number of function evaluations exceeded --- " \
              "increase maxfun argument.\n")
    return


def brute(func, ranges, args=(), Ns=20, full_output=0, finish=fmin):
    """Minimize a function over a given range by brute force.

    :Parameters:

        func : callable ``f(x,*args)``
            Objective function to be minimized.
        ranges : tuple
            Each element is a tuple of parameters or a slice object to
            be handed to ``numpy.mgrid``.
        args : tuple
            Extra arguments passed to function.
        Ns : int
            Default number of samples, if those are not provided.
        full_output : bool
            If True, return the evaluation grid.

    :Returns: (x0, fval, {grid, Jout})

        x0 : ndarray
            Value of arguments to `func`, giving minimum over the grid.
        fval : int
            Function value at minimum.
        grid : tuple
            Representation of the evaluation grid.  It has the same
            length as x0.
        Jout : ndarray
            Function values over grid:  ``Jout = func(*grid)``.

    :Notes:

        Find the minimum of a function evaluated on a grid given by
        the tuple ranges.

    """
    N = len(ranges)
    if N > 40:
        raise ValueError("Brute Force not possible with more " \
              "than 40 variables.")
    lrange = list(ranges)
    for k in range(N):
        if type(lrange[k]) is not type(slice(None)):
            if len(lrange[k]) < 3:
                lrange[k] = tuple(lrange[k]) + (complex(Ns),)
            lrange[k] = slice(*lrange[k])
    if (N==1):
        lrange = lrange[0]

    def _scalarfunc(*params):
        params = squeeze(asarray(params))
        return func(params,*args)

    vecfunc = vectorize(_scalarfunc)
    grid = mgrid[lrange]
    if (N==1):
        grid = (grid,)
    Jout = vecfunc(*grid)
    Nshape = shape(Jout)
    indx = argmin(Jout.ravel(),axis=-1)
    Nindx = zeros(N,int)
    xmin = zeros(N,float)
    for k in range(N-1,-1,-1):
        thisN = Nshape[k]
        Nindx[k] = indx % Nshape[k]
        indx = indx / thisN
    for k in range(N):
        xmin[k] = grid[k][tuple(Nindx)]

    Jmin = Jout[tuple(Nindx)]
    if (N==1):
        grid = grid[0]
        xmin = xmin[0]
    if callable(finish):
        vals = finish(func,xmin,args=args,full_output=1, disp=0)
        xmin = vals[0]
        Jmin = vals[1]
        if vals[-1] > 0:
            print("Warning: Final optimization did not succeed")
    if full_output:
        return xmin, Jmin, grid, Jout
    else:
        return xmin


def main():
    import time

    times = []
    algor = []
    x0 = [0.8,1.2,0.7]
    print("Nelder-Mead Simplex")
    print("===================")
    start = time.time()
    x = fmin(rosen,x0)
    print(x)
    times.append(time.time() - start)
    algor.append('Nelder-Mead Simplex\t')

    print()
    print("Powell Direction Set Method")
    print("===========================")
    start = time.time()
    x = fmin_powell(rosen,x0)
    print(x)
    times.append(time.time() - start)
    algor.append('Powell Direction Set Method.')

    print()
    print("Nonlinear CG")
    print("============")
    start = time.time()
    x = fmin_cg(rosen, x0, fprime=rosen_der, maxiter=200)
    print(x)
    times.append(time.time() - start)
    algor.append('Nonlinear CG     \t')

    print()
    print("BFGS Quasi-Newton")
    print("=================")
    start = time.time()
    x = fmin_bfgs(rosen, x0, fprime=rosen_der, maxiter=80)
    print(x)
    times.append(time.time() - start)
    algor.append('BFGS Quasi-Newton\t')

    print()
    print("BFGS approximate gradient")
    print("=========================")
    start = time.time()
    x = fmin_bfgs(rosen, x0, gtol=1e-4, maxiter=100)
    print(x)
    times.append(time.time() - start)
    algor.append('BFGS without gradient\t')


    print()
    print("Newton-CG with Hessian product")
    print("==============================")
    start = time.time()
    x = fmin_ncg(rosen, x0, rosen_der, fhess_p=rosen_hess_prod, maxiter=80)
    print(x)
    times.append(time.time() - start)
    algor.append('Newton-CG with hessian product')


    print()
    print("Newton-CG with full Hessian")
    print("===========================")
    start = time.time()
    x = fmin_ncg(rosen, x0, rosen_der, fhess=rosen_hess, maxiter=80)
    print(x)
    times.append(time.time() - start)
    algor.append('Newton-CG with full hessian')

    print()
    print("\nMinimizing the Rosenbrock function of order 3\n")
    print(" Algorithm \t\t\t       Seconds")
    print("===========\t\t\t      =========")
    for k in range(len(algor)):
        print(algor[k], "\t -- ", times[k])

if __name__ == "__main__":
    main()



#PBS -N LongLangevin
#PBS -q long
#PBS -l nodes=1:opteron4

#! /usr/bin/env python

"""longLangevin.py - test the Langevin dynamics.

Usage: python longLangevin.py

Tests Langevin dynamics using the EMT Copper potential.  It takes a while
since it is a statistical test of the temperature control.
"""
from __future__ import print_function

import sys, time
import numpy as np
from asap3.testtools import ReportTest
from ase import *
from asap3 import *
from asap3.md.langevin import Langevin
from asap3.md.verlet import VelocityVerlet
from ase.lattice.cubic import FaceCenteredCubic
#from Scientific.Statistics import standardDeviation, variance
#from Scientific.Functions.LeastSquares import leastSquaresFit

nsteps = 1000000
#nsteps = 5000
nprint = 50000
nequil = 100000
nminor = 25
nequilprint = 200
tolerance = 0.01
timestep = 5 # fs
frict = 0.001
temp = 100 # K
repeats = 5

# Set up atoms in a regular simple-cubic lattice.
initial = FaceCenteredCubic(directions=[[1,0,0],[0,1,0],[0,0,1]],
                            size = (1,5,5), symbol="Cu", latticeconstant=3.5,
                            pbc=False, debug=0)
initial.set_calculator(EMT())
    
ReportTest("Number of atoms", len(initial), 100, 0)

# Make a small perturbation of the momenta
initial.set_momenta(1e-6 * np.random.normal(size=[len(initial), 3]))
print("Initializing (1) ...")
predyn = VelocityVerlet(initial, 0.5)
predyn.run(2500)

initial2 = FaceCenteredCubic(directions=[[1,0,0],[0,1,0],[0,0,1]],
                             size = (4,4,4), symbol="Cu", debug=0)
initial2.set_calculator(EMT())
# Make a small perturbation of the momenta
initial2.set_momenta(1e-6 * np.random.normal(size=[len(initial2), 3]))
print("Initializing (2) ...")
predyn = VelocityVerlet(initial2, 0.5)
predyn.run(2500)


class TemperatureMonitor:
    def __init__(self, report):
        self.report = report
        self.reports = []

    def New(self, name, atoms, temperature, pas):
        self.name = "%s (pass %d)" % (name, pas+1)
        self.atoms = atoms
        self.n = 0
        self.ncall = 0
        self.sum = 0.0
        self.temperature = temperature

    def Update(self):
        self.ncall += 1
        T = self.atoms.get_kinetic_energy() / (1.5 * units.kB * len(self.atoms))
        self.sum += T
        self.n += 1
        if self.ncall % self.report == 0:
            tavg = self.sum / self.n
            print("%5d  T = %7.2f K  T_avg = %7.2f" % (self.ncall, T, tavg))
                
    def Check(self):
        T = self.sum / self.n
        result = "%s   T = %.2f K   (expected %.2f K)" % (self.name, T,
                                                          self.temperature)
        print(result)
        self.reports.append(result)
        ReportTest(self.name, T, self.temperature, 0.01*self.temperature)
        
    def Report(self):
        for r in self.reports:
            print(r)

def targetfunc(params, x):
    return params[0] * exp(-params[1] * x) + params[2]

def targetfunc2(x, a, b, c):
    return a * np.exp(-b * x) + c
    
def equilibrate(atoms, dyn, temp):
    print("Equilibrating ...")
    tstart = time.time()
    for i in xrange(1,nequil/nminor+1):
        dyn.run(nminor)
        if i % nequilprint == 0:
            ekin = atoms.get_kinetic_energy() / len(atoms)
            temperature = 2.0/3.0 * ekin
            print("%.6f  T = %.6f (goal: %f)" % \
                      (ekin, temperature, temp, ))
    tequil = time.time() - tstart
    print("This took %.2f minutes." % (tequil / 60))
    print("Taking data - this takes", repeats*nsteps/nequil, "times longer!")

monitor = TemperatureMonitor(nprint)

atoms = Atoms(initial)
atoms.set_calculator(EMT())
dyn = Langevin(atoms, timestep*units.fs, temp*units.kB, 0.001)
equilibrate(atoms, dyn, units.kB*temp)
dyn.attach(monitor.Update, 5)
for i in range(repeats):
    monitor.New("Free boundaries, fixcm=True", atoms, temp, i)
    dyn.run(nsteps)
    monitor.Check()

atoms = Atoms(initial)
atoms.set_calculator(EMT())
dyn = Langevin(atoms, timestep*units.fs, temp*units.kB, 0.001, fixcm=False)
equilibrate(atoms, dyn, units.kB*temp)
dyn.attach(monitor.Update, 5)
for i in range(repeats):
    monitor.New("Free boundaries, fixcm=False", atoms, temp, i)
    dyn.run(nsteps)
    monitor.Check()

atoms = Atoms(initial2)
atoms.set_calculator(EMT())
dyn = Langevin(atoms, timestep*units.fs, temp*units.kB, 0.001)
equilibrate(atoms, dyn, units.kB*temp)
dyn.attach(monitor.Update, 5)
for i in range(repeats):
    monitor.New("Periodic boundaries, fixcm=True", atoms, temp, i)
    dyn.run(nsteps)
    monitor.Check()

atoms = Atoms(initial2)
atoms.set_calculator(EMT())
dyn = Langevin(atoms, timestep*units.fs, temp*units.kB, 0.001, fixcm=False)
equilibrate(atoms, dyn, units.kB*temp)
dyn.attach(monitor.Update, 5)
for i in range(repeats):
    monitor.New("Periodic boundaries, fixcm=False", atoms, temp, i)
    dyn.run(nsteps)
    monitor.Check()

monitor.Report()
ReportTest.Summary()





"""Check that ASE.calculators.emt and asap3.EMT give the same."""

from __future__ import print_function
from ase.lattice.compounds import *
from ase.lattice.cubic import *
from ase.calculators.emt import EMT as EMT_ASE
from asap3 import EMT as EMT_ASAP
from asap3.testtools import ReportTest
import numpy as np

elements = ("Ni", "Cu", "Pd", "Ag", "Pt")

for e1 in elements:
    for e2 in elements:
        atoms = FaceCenteredCubic(directions=[[1,0,0],[0,1,0],[0,0,1]], size=(1,1,2),
                                  symbol=e1, pbc=(1,0,1), debug=0)
        atoms.set_calculator(EMT_ASE(asap_cutoff=True))
        e_e1_ase = atoms.get_potential_energy()
        atoms.set_calculator(EMT_ASAP())
        e_e1_asap = atoms.get_potential_energy()
        natoms = len(atoms)

        print("{0} energy (ASE) \t{1:.5f}".format(e1, e_e1_ase/natoms))
        print("{0} energy (ASAP)\t{1:.5f}".format(e1, e_e1_asap/natoms))

        atoms = FaceCenteredCubic(directions=[[1,0,0],[0,1,0],[0,0,1]], size=(1,1,2),
                                  symbol=e2, pbc=(1,0,1), debug=0)
        atoms.set_calculator(EMT_ASE(asap_cutoff=True))
        e_e2_ase = atoms.get_potential_energy()
        atoms.set_calculator(EMT_ASAP())
        e_e2_asap = atoms.get_potential_energy()

        print("{0} energy (ASE) \t{1:.5f}".format(e2, e_e2_ase/natoms))
        print("{0} energy (ASAP)\t{1:.5f}".format(e2, e_e2_asap/natoms))

        atoms = L1_2(directions=[[1,0,0],[0,1,0],[0,0,1]], size=(1,1,2),
                     symbol=(e1, e2), latticeconstant=3.95, pbc=(1,0,1), 
                     debug=0)
        
        atoms.set_calculator(EMT_ASE(asap_cutoff=True))
        e_alloy_ase = atoms.get_potential_energy() - (2*e_e1_ase + 6*e_e2_ase)/8
        atoms.set_calculator(EMT_ASAP())
        e_alloy_asap = atoms.get_potential_energy() - (2*e_e1_asap + 6*e_e2_asap)/8

        print("Alloy energy (ASE) \t{0:.5f}".format(e_alloy_ase/natoms))
        print("Alloy energy (ASAP)\t{0:.5f}".format(e_alloy_asap/natoms))
        ReportTest("{0}{1}_3 alloy energy".format(e1, e2), e_alloy_ase, e_alloy_asap, 1e-4)

ReportTest.Summary()




#!/usr/bin/env python
"""
Tests that polyhedral template matching works on the simplest possible system.

Name: ptm.py

Description: Part of the Asap test suite.  Tests the analysis.ptm module

Usage: python ptm.py

Expected result: The output should end with 'ALL TESTS SUCCEEDED'.
"""
from __future__ import print_function

from ase.lattice.cubic import FaceCenteredCubic, BodyCenteredCubic
from ase.lattice.hexagonal import HexagonalClosedPacked
from ase.build import bulk
from asap3.testtools import ReportTest
from asap3.analysis import PTM
import numpy as np
#from ase.visualize import view

atoms = FaceCenteredCubic("Cu", size=(7,7,7), pbc=False)
x = atoms.get_positions()
#atoms.set_positions(x + 0.01 * np.random.standard_normal(x.shape))

data = PTM(atoms, rmsd_max=0.1, calculate_strains=True)
print("DONE")
cnt = np.bincount(data['structure'])
print(cnt)
print(data['info'])

ReportTest("Number of FCC atoms", cnt[1], 6*6*6*4, 0)
ReportTest("Number of non-classified atoms", cnt[0], len(atoms) - 6*6*6*4, 0)

data = PTM(atoms, rmsd_max=0.1, calculate_strains=True, quick=True)
cnt = np.bincount(data['structure'])
print(cnt)
print(data['info'])

ReportTest("Number of FCC atoms", cnt[1], 6*6*6*4, 0)
ReportTest("Number of non-classified atoms", cnt[0], len(atoms) - 6*6*6*4, 0)

data = PTM(atoms, rmsd_max=0.1, target_structures=('fcc', 'hcp'))
cnt = np.bincount(data['structure'])
print(cnt)
print(data['info'])

ReportTest("Number of FCC atoms", cnt[1], 6*6*6*4, 0)
ReportTest("Number of non-classified atoms", cnt[0], len(atoms) - 6*6*6*4, 0)

atoms = BodyCenteredCubic("Fe", size=(7,7,7), pbc=(True,True,False))
data = PTM(atoms, rmsd_max=0.001)
cnt = np.bincount(data['structure'])
print(cnt)
print(data['info'])

ReportTest("Number of BCC atoms", cnt[3], 7*7*5*2, 0)
ReportTest("Number of non-classified atoms", cnt[0], len(atoms) - 7*7*5*2, 0)

atoms = HexagonalClosedPacked("Mg", size=(7,7,7), pbc=(True,True,False))
data = PTM(atoms, rmsd_max=0.01)  # Need a slightly larger rmsd as c/a deviates from ideal.
cnt = np.bincount(data['structure'])
print(cnt)
print(data['info'])

ReportTest("Number of HCP atoms", cnt[2], 7*7*6*2, 0)
ReportTest("Number of non-classified atoms", cnt[0], len(atoms) - 7*7*6*2, 0)
#atoms.set_tags(data['structure'])
#view(atoms)

atoms = bulk("Mg", orthorhombic=True)
atoms = atoms.repeat((7,7,7))
atoms.set_pbc((True, True, False))
data = PTM(atoms, rmsd_max=0.01)  # Need a slightly larger rmsd as c/a deviates from ideal.
cnt = np.bincount(data['structure'])
print(cnt)
print(data['info'])

ReportTest("Number of HCP atoms", cnt[2], 7*7*6*4, 0)
ReportTest("Number of non-classified atoms", cnt[0], len(atoms) - 7*7*6*4, 0)

# Make a combined system

atoms = bulk("Mg", orthorhombic=True)
atoms = atoms.repeat((7,4,7))

atoms2 = bulk("Fe", orthorhombic=True).repeat((3,3,3))
uc1 = atoms.get_cell()
uc2 = atoms2.get_cell()
factor1 = uc1[0,0] / uc2[0,0]
factor2 = uc1[1,1] / uc2[1,1]
uc2 = uc2 * np.array((factor1, factor2, 0.5*(factor1+factor2)))
atoms2.set_cell(uc2, scale_atoms=True)
# Combine unit cell, stack BCC atoms on top of HCP atoms.
uc3 = np.array(uc1)
dz = uc3[2,2]
uc3[2,2] += uc2[2,2]
atoms.set_cell(uc3, scale_atoms=False)
atoms2.set_positions(atoms2.get_positions() + np.array((0.1,0.1,dz))[np.newaxis,:])
atoms.extend(atoms2)

data = PTM(atoms, rmsd_max=0.005)
cnt = np.bincount(data['structure'])
print(cnt)
print(data['info'])

ReportTest("Number of HCP atoms", cnt[2], 7*4*6*4, 0)
ReportTest("Number of BCC atoms", cnt[3], 3*3*1*2, 0)
ReportTest("Number of FCC atoms", cnt[1], 0, 0)
ReportTest("All classes of atoms", cnt[:4].sum(), len(atoms), 0)

#atoms.set_tags(data['structure'])
#view(atoms)

ReportTest.Summary()



import numpy as npy

from asap3.optimize.optimize import Optimizer


class MDMin(Optimizer):
    def __init__(self, atoms, restart=None, logfile='-', trajectory=None,
                 dt=None):
        Optimizer.__init__(self, atoms, restart, logfile, trajectory)

        if dt is not None:
            self.dt = dt

    def initialize(self):
        self.dt = 0.1

    def read(self):
        v, self.dt = self.load()
        atoms.set_velocities(v)
        
    def step(self, f):
        atoms = self.atoms

        v = atoms.get_velocities()
        if v is None:
            v = npy.zeros((len(atoms), 3))
        else:
            v += 0.5 * self.dt * f
            # Correct velocities:
            vf = npy.vdot(v, f)
            if vf < 0.0:
                v[:] = 0.0
            else:
                v[:] = f * vf / npy.vdot(f, f)

        v += 0.5 * self.dt * f
        r = atoms.get_positions()
        atoms.set_positions(r + self.dt * v)
        atoms.set_velocities(v)
        self.dump((v, self.dt))



"""Test periodic boundary conditions in parallel simulations."""
from __future__ import print_function

from asap3 import *
from asap3.analysis.localstructure import RestrictedCNA, CoordinationNumbers
from asap3.testtools import ReportTest
from asap3.mpi import world
from ase.lattice import bulk
import numpy as np

debug = 0
if debug == 1:
    DebugOutput("parallel%d.log", nomaster=True)
elif debug == 2:
    time.sleep(world.rank)
    print("PID:", os.getpid())
    time.sleep(20)

ismaster = world.rank == 0
isparallel = world.size != 1
if world.size == 1:
    cpulayout = None
elif world.size == 2:
    cpulayout = [2,1,1]
elif world.size == 3:
    cpulayout = [1,3,1]
elif world.size == 4:
    cpulayout = [2,1,2]

pbc_list = [True,
            False,
            (True, True, False),
            (False, True, True),
            (True, False, True)]

def makesystem():
    atoms = bulk('Cu')
    atoms = atoms.repeat((15, 15, 15))
    if cpulayout:
        atoms = atoms.repeat(cpulayout)
    del atoms[87]   #Create point defect
    atoms.center(vacuum = 10.0)
    return atoms

seratoms = makesystem()
seratoms.set_calculator(EMT())
old_energy = seratoms.get_potential_energy()

for pbc in pbc_list:
    # Make parallel nb list
    if ismaster:
        atoms = makesystem()
        atoms.set_pbc(pbc)
    else:
        atoms = None
    if isparallel:
        atoms = MakeParallelAtoms(atoms, cpulayout)
    natoms = atoms.get_number_of_atoms()
    atoms.set_calculator(EMT())
    energy = atoms.get_potential_energy()
    ReportTest("PBC={0}".format(str(pbc)), energy, old_energy, 1e-6)

ReportTest.Summary()



import numpy as np


def plane_equation(centre, h, cell):

    vertices = [cell[e] for e in h]
    p = vertices[0]

    n = np.cross(vertices[1] - p, vertices[2] - p)
    n /= np.linalg.norm(n)

    side = np.dot(n, centre - p)
    if side < 0:
        n = -n
        side = np.dot(n, centre - p)
        if side < 0:
            raise Exception("n is not a normal vector")
    return (p, n)

def lies_in_cell(bounding_planes, point):

    for (p, n) in bounding_planes:
        side = np.dot(n, point - p)
        if side < 0:
            return False
    return True




from __future__ import print_function
import numpy as np
from asap3 import *
from ase.lattice.cubic import FaceCenteredCubic
from asap3.testtools import ReportTest

#set_verbose(1)
atoms = FaceCenteredCubic(symbol='Cu', size=(4,4,4))
atoms.set_calculator(EMT())
uc = atoms.get_cell()
#vals = (0.0, 0.02, 0.04, 0.06, 0.08, 0.1, 0.2, 0.3)
#vals = linspace(-0.1444, 0.1444, 20)
vals = np.linspace(0, 0.2, 20)

res = []
for n,i in enumerate(vals):
    uc[0,2] = i
    atoms.set_cell(uc, scale_atoms=True)
    epot = atoms.get_potential_energy()
    print(n, i, epot)
    res.append(epot)

poly = np.polyfit(vals, res, 2)
fits = np.polyval(poly, vals)

rms = np.sqrt((fits - res) * (fits - res))
print(rms)
maxrms = rms.max()
ReportTest("Worse fit", maxrms, 0.0, 1e-5)
ReportTest.Summary()



    




from __future__ import print_function
from asap3 import *
from ase.lattice.cubic import FaceCenteredCubic
from asap3.testtools import ReportTest

atoms = FaceCenteredCubic(size=(5,5,5), symbol="Cu")
pot = EMT()
atoms.set_calculator(pot)

for i in (1, 2):
    print("*** Pass", i)
    ReportTest("Energy required", pot.calculation_required(atoms, ["energy"]), 1, 0)
    ReportTest("Forces required", pot.calculation_required(atoms, ["forces"]), 1, 0)
    ReportTest("Stress required", pot.calculation_required(atoms, ["stress"]), 1, 0)
    ReportTest("Magmom required", pot.calculation_required(atoms, ["magmoms"]), 1, 0)
    e = atoms.get_potential_energy()
    ReportTest("Energy not required", pot.calculation_required(atoms, ["energy"]), 0, 0)
    ReportTest("Forces required (II)", pot.calculation_required(atoms, ["forces"]), 1, 0)
    f = atoms.get_forces()
    ReportTest("Energy not required (II)", pot.calculation_required(atoms, ["energy"]), 0, 0)
    ReportTest("Forces not required", pot.calculation_required(atoms, ["forces"]), 0, 0)
    ReportTest("Energy or forces not required",
               pot.calculation_required(atoms, ["energy", "forces"]), 0, 0)
    ReportTest("Energy or stress required",
               pot.calculation_required(atoms, ["energy", "stress"]), 1, 0)
    s = atoms.get_stress()
    ReportTest("Stress not required", pot.calculation_required(atoms, ["stress"]), 0, 0)

    r = atoms.get_positions()
    r[0,0] += 0.1
    atoms.set_positions(r)

ReportTest.Summary()




from __future__ import print_function
from ase.constraints import FixAtoms as ASE_FixAtoms
from ase.constraints import Filter as ASE_Filter
import numpy as np

class ConstraintMixin:
    """Mixin class FixAtoms and Filter"""
         
    def prepare_for_asap(self, atoms):
        """Prepare this constraint for optimized Asap dynamics
        
        This function must be called once the atoms are known by
        all dynamics supporting parallel MD.
        """
        # Store the arrays of the atoms, not the atoms, to prevent cyclic references.
        self.atoms_arrays = atoms.arrays
        assert self.indexname not in self.atoms_arrays
        idx = self.index
        self.asap_ready = True
        if getattr(atoms, "parallel", False):
            self.storable = False
        self.index = idx
        del self._index
        assert self.indexname in self.atoms_arrays
        
    def set_index(self, idx):
        if self.asap_ready:
            natoms = len(self.atoms_arrays['positions'])
            if idx.dtype == bool:
                # Boolean - must be a mask
                assert len(idx) == natoms
            else:
                # Must be a list of indices.  Convert to a mask
                idx2 = np.zeros(natoms, bool)
                idx2[idx] = True
                idx = idx2
            self.atoms_arrays[self.indexname] = idx
        else:
            self._index = idx
            
    def get_index(self):
        if self.asap_ready:
            return self.atoms_arrays[self.indexname]
        else:
            return self._index

    def todict(self):
        "Convert to dictionary for storage in e.g. a Trajectory object."
        if not self.storable:
            raise RuntimeError(
                "Cannot store asap3.{0} constraint in parallel simulations.".format(self.dictname))
        idx = self.index
        if idx.dtype == bool:
            return {'name': self.dictname,
                    'kwargs': {'mask': idx}}
        else:
            return {'name': self.dictname,
                    'kwargs': {'indices': idx}}
                
class FixAtoms(ConstraintMixin, ASE_FixAtoms, object):
    dictname = 'FixAtoms'  # Name used when converting to dictionary.
    def __init__(self, indices=None, mask=None):
        self.pre_init()
        ASE_FixAtoms.__init__(self, indices, mask)
        
    def pre_init(self):
        self.indexname = "FixAtoms_index"
        self.asap_ready = False
        self.storable = True
        
    def copy(self):
        if self.index.dtype == bool:
            return FixAtoms(mask=self.index.copy())
        else:
            return FixAtoms(indices=self.index.copy())
        
    def __get_state__(self):
        return {'data': self.index,
                'version': 1}
        
    def __set_state__(self, state):
        try:
            assert(state['version'] == 1)
        except KeyError:
            print(state)
            raise
        self.pre_init()
        self.index = state['data']
        
    index = property(ConstraintMixin.get_index, ConstraintMixin.set_index)
    
    
class Filter(ConstraintMixin, ASE_Filter, object):
    dictname = 'Filter'  # Name used when converting to dictionary.
    def __init__(self, atoms, indices=None, mask=None):
        """Filter atoms.

        This filter can be used to hide degrees of freedom in an Atoms
        object.

        Parameters
        ----------
        indices : list of int
           Indices for those atoms that should remain visible.
        mask : list of bool
           One boolean per atom indicating if the atom should remain
           visible or not.
        """

        self.pre_init()
        ASE_Filter.__init__(self, atoms, indices, mask)
        self.prepare_for_asap(atoms)
        
    def pre_init(self):
        self.indexname = "Filter_index"
        self.asap_ready = False
        self.storable = True

    index = property(ConstraintMixin.get_index, ConstraintMixin.set_index)  
    
def check_asap_constraints(atoms, allowed=None):
    """Check that atoms only have allowed constraints.  Return True if so, otherwise False.
    
    An optional second parameter can be a tuple of allowed constraints.
    """
    if allowed is None:
        allowed = (FixAtoms,)
        
    if len(atoms.constraints) == 0:
        return True
    if len(atoms.constraints) > 1:
        return False
    return isinstance(atoms.constraints[0], allowed)




from asap3.Internal.Builtins import timing_results
import sys, string
from numpy import array

def _reporttiming(file, name, data, all=1):
    if (data[0]+data[2]) != 0:
        ptot = "%.0f%%" % (100*(data[1]+data[3])/(data[0]+data[2]))
    else:
        ptot = "-"
    if data[0] != 0:
        pself = "%.0f%%" % (100*data[1]/data[0])
    else:
        pself = "-"
    if data[2] != 0:
        pch = "%.0f%%" % (100*data[3]/data[2])
    else:
        pch = "-"
    if all:
        xtotal = "Total: %.1f / %.1f sec (%s)." % (data[1]+data[3],
                                                   data[0]+data[2], ptot,)
    else:
        xtotal = ""
    xself = "Self: %.1f / %.1f sec (%s)." % (data[1], data[0], pself)
    if all:
        xch = "Children: %.1f / %.1f sec (%s)." % (data[3], data[2], pch)
    else:
        xch = ""
    if len(name) > 25:
        file.write("%s\n%-26s n=%-4d %-33s %-33s %s\n" % (name+":", "",
                                                          data[4], xtotal,
                                                          xself, xch))
    else:
        file.write("%-26s n=%-4d %-33s %-33s %s\n" % (name+":", data[4], 
                                                      xtotal,xself, xch))
    if all and data[5] is not None:
        keys = data[5].keys()
        keys.sort()
        for master in keys:
            mdat = data[5][master]
            file.write("     for %-45s n=%-4d Tot: %.1f / %.1f  Self: %.1f / %.1f  Ch: %.1f / %.1f\n"
                       % (master+":", mdat[4], mdat[1]+mdat[3],
                          mdat[0]+mdat[2], mdat[1], mdat[0], mdat[3], mdat[2]))
            
def report_timing(fil = sys.stdout, header = "\nASAP Timing results:\n"):
    if type(fil) == type("string"):
        fil = open(fil, "w")
    fil.write(header+"\n")
    t = timing_results()
    if t is None:
        fil.write("No timing data!  Compile with -DTIMING to enable it.\n")
    else:
        _reporttiming(fil, "Global", t["Global"])
        _reporttiming(fil, "Timing overhead", t["Timing overhead"])
        del t["Global"], t["Timing overhead"]
        fil.write("\nFunctions and methods:\n")
        keys = t.keys()
        keys.sort()
        for l in keys:
            _reporttiming(fil, l, t[l])
        fil.write("\nClasses:\n")
        cls = {}
        for l in keys:
            words = string.split(l, "::")
            if len(words) == 2:
                try:
                    cls[words[0]].append(t[l][:5])
                except KeyError:
                    cls[words[0]] = [t[l][:5]]
        keys = cls.keys()
        keys.sort()
        for l in keys:
            v = sum(array(cls[l]))
            _reporttiming(fil, l, v, all=0)




from __future__ import print_function
from asap3 import *
from ase.lattice.cubic import *

atoms = FaceCenteredCubic(size=(10, 10, 10), symbol="Cu")
z = atoms.get_atomic_numbers()
z[7]=8
atoms.set_atomic_numbers(z)

try:
    atoms.set_calculator(EMT())
except AsapError:
    print("Test 1 passed.")
else:
    print("Test 1 failed.")
    
atoms = FaceCenteredCubic(size=(1, 10, 10), symbol="Cu")
atoms.set_calculator(EMT())
try:
    e = atoms.get_potential_energy()
except AsapError:
    print("Test 2 passed.")
else:
    print("Test 2 failed.")
    

atoms = FaceCenteredCubic(size=(10, 10, 10), symbol="Cu")
r = atoms.get_positions()
r[10] = r[11]
atoms.set_positions(r)
atoms.set_calculator(EMT())
try:
    e = atoms.get_potential_energy()
except AsapError:
    print("Test 3 passed.")
else:
    print("Test 3 failed.")
    


# Copyright (C) 2014 Simon H. Brodersen and Center for Individual
# Nanoparticle Functionality, Department of Physics, Technical
# University of Denmark.  Email: schiotz@fysik.dtu.dk
#
# This file is part of Asap version 3.
#
# This program is free software: you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public License
# version 3 as published by the Free Software Foundation.  Permission
# to use other versions of the GNU Lesser General Public License may
# granted by Jakob Schiotz or the head of department of the
# Department of Physics, Technical University of Denmark, as
# described in section 14 of the GNU General Public License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# and the GNU Lesser Public License along with this program.  If not,
# see <http://www.gnu.org/licenses/>.

import numpy as np

from asap3 import FullNeighborList, CoordinationNumbers
from asap3.analysis.localstructure import GuessLatticeConstant

def GetDiameter(atoms, N=500, all=False):
    # Based on the spiral points method by E. Saff and A. Kuijlaars,
    # The Mathematical Intelligencer (1997)
    h = 2.0*np.arange(N)/(N - 1) - 1
    theta = np.arccos(h)
    phi = np.zeros(N)
    for i in range(1, N - 1):
        phi[i] = (phi[i - 1] + 3.6/np.sqrt(N*(1 - h[i]**2)))# % 2*np.pi

    directions = np.empty((N,3))
    directions[:,0] = np.sin(theta) * np.cos(phi)
    directions[:,1] = np.sin(theta) * np.sin(phi)
    directions[:,2] = np.cos(theta)

    # Project the positions onto the grid and return the mean diameter
    p = atoms.get_positions()
    r = p - p.mean(axis=0)

    diameters = np.zeros(N)
    for i, d in enumerate(directions):
        d = np.dot(r, d)
        diameters[i] = d.max() - d.min()

    if all:
        return diameters
    else:
        return diameters.mean()

def GetLayerNumbers(atoms, rNearest=None, maxlayers=-1):
    if not rNearest:
        rNearest = GuessLatticeConstant(atoms) * 0.707
    c_nb = rNearest * 1.17 #Nearest neighbor cutoff
    c_sh = rNearest * 1.50 #Surface atoms cutoff 

    nb = FullNeighborList(c_nb, atoms)
    cn = CoordinationNumbers(atoms, c_nb)

    rad = GetDiameter(atoms) / 2.0
    pos = atoms.get_positions()
    cop = pos.mean(axis=0)
    r = np.sqrt(np.sum((pos - cop)**2, axis=1))

    tags = ((cn < 11) & (r > (rad - c_sh))).astype(int)
    left = len(atoms) - tags.sum()

    n = 1
    while left:
        for i in np.arange(len(tags))[tags == n]:
            nbi = nb[i]
            ids = nbi[tags[nbi] == 0]
            tags[ids] = n + 1
            left -= len(ids)
        n += 1
        if n > maxlayers and maxlayers > 0:
            tags[tags == 0] = n
            break

    return tags




from __future__ import print_function
import numpy as np

from asap3 import RGL

def RGLFit(elements, parameters, *args, **kwargs):
    n = len(elements)
    p = np.zeros((n, n))
    q = np.zeros((n, n))
    a = np.zeros((n, n))
    xi = np.zeros((n, n))
    r0 = np.zeros((n, n))

    elementmap = {}
    for i, symbol in enumerate(elements):
        elementmap[symbol] = i

    for key in parameters.keys():
        if len(key) == 1:
            i = j = elementmap[key[0]]
        elif len(key) == 2:
            i = elementmap[key[0]]
            j = elementmap[key[1]]
        else:
            raise KeyError('Number of elements in the key should be one or two.')

        p[i,j] = parameters[key][0]
        q[i,j] = parameters[key][1]
        a[i,j] = parameters[key][2]
        xi[i,j] = parameters[key][3]
        r0[i,j] = parameters[key][4]

        a1 = (np.log(np.sqrt(12) * a[i,j] * p[i,j] / (xi[i,j] * q[i,j])) /
              (p[i,j] - q[i,j]) + 1) * r0[i,j]
        if not 1.0 < a1 < 10.0:
            raise ValueError('Unreasonable parameters: a1 = %.3f' % (a1,))

    return RGL(elements, p, q, a, xi, r0, *args, **kwargs)

if __name__ == '__main__':
    # This script is an example of how to fit RGL parameters.
    import sys

    from asap3.Tools.ParameterOptimization import ParameterOptimization
    from asap3.Tools.OptimizationDatabase import get_data

    initparameters = {('Cu','Cu'): [10.55, 2.43, 0.0894, 1.2799, 2*1.28, 3.61*1.75, 3.61*0.1]}

    varparameters = {('Cu','Cu'): [True, True, True, True, True, False, False]}

    fitquantities = [('lattice_constant_a', 'fcc', 'Cu', get_data('Cu', 'a'), 0.005),
                     ('cohesive_energy', 'fcc', 'Cu', get_data('Cu', 'Ecoh'), 0.01),
                     ('bulk_modulus', 'fcc', 'Cu', get_data('Cu', 'B'), 0.01),
                     ('elastic_constant_C11', 'fcc', 'Cu', get_data('Cu', 'C11'), 0.01),
                     ('elastic_constant_C12', 'fcc', 'Cu', get_data('Cu', 'C12'), 0.01),
                     ('elastic_constant_C44', 'fcc', 'Cu', get_data('Cu', 'C44'), 0.01),
                     ]

    latticeconstants = [['fcc', 'Cu', 4.00]]

    opt = ParameterOptimization(['Cu'], RGLFit, initparameters, varparameters,
                                fitquantities, latticeconstants, debug=False)
    (optparam, optvalues) = opt.fit(log=sys.stdout)

    print("Initial parameters:", initparameters)
    print("Optimal parameters:", optparam)
    #print ""
    #print "Fitting values", fitquantities
    #print "Optimal values", optvalues



"""Check that energy is correct even after wrapping through periodic boundary conditions.
"""
from __future__ import print_function

from ase.lattice.cubic import FaceCenteredCubic
from asap3 import *
from asap3.testtools import *
import random

ref_atoms = FaceCenteredCubic(size=(7,7,7), symbol="Cu", pbc=(True, False, True))
ref_atoms.set_calculator(EMT())

ref_energy = ref_atoms.get_potential_energy()
ref_energies = ref_atoms.get_potential_energies()
ref_forces = ref_atoms.get_forces()

passes = 5
for ps in range(passes):
    print("Pass", ps, "of", passes)

    atoms = ref_atoms.copy()
    atoms.set_calculator(EMT())
    nat = random.randint(0, len(atoms))
    assert nat < len(atoms)
    pos0 = atoms[nat].position
    cell = atoms.get_cell()
    for d in range(1,4):
        for dx in (-d, 0, d):
            #for dy in (-d, 0, d):
            for dy in (0,):
                for dz in (-d, 0 ,d):
                    deltar = dx * cell[0] + dy * cell[1] + dz * cell[2]
                    atoms[nat].position = pos0 + deltar
                    label = "(%2d, %2d, %2d)" % (dx, dy, dz)
                    ReportTest("Pot. energy   "+label,
                               atoms.get_potential_energy(), ref_energy, 1e-6)
                    de = (atoms.get_potential_energies() - ref_energies)
                    ReportTest("Pot. energies "+label, de.max(), 0.0, 1e-6)
                    df = (atoms.get_forces() - ref_forces)
                    ReportTest("Forces        "+label, df.max(), 0.0, 1e-6)
                
ReportTest.Summary()




# DisplacementField.py  --  Implements displacement fields

# Copyright (C) 1999 CAMP and Jakob Schiotz <schiotz@fysik.dtu.dk>

"""Defines the DisplacementField class.

Displacement fields are returned e.g. by the `Setup.Dislocation`
modules, they are vector fields (see `VectorFunction`) and can be
applied to a lattice using the `apply_to()` method.  Two displacement
fields can be added or subtracted, and a field can be
multiplied/divided by a scalar.  """

__docformat__ = "restructuredtext en"

import numpy

class VectorFunction:
    """Implements vector function objects.

    Function objects are callable objects (i.e. they behave like
    functions), but they can be added and multiplied with scalars
    and with themselves.
    """
    
    def __init__(self,expr):
        self._expr = expr

    def __call__(self, *args, **kwargs):
        return self._expr(*args, **kwargs)

    def __add__(self,other):
        if callable(other):
            return self.__class__(lambda x,other=other,oldfunc=self : oldfunc(x)+other(x))
        else:
            return self.__class__(lambda x,const=other,oldfunc=self : oldfunc(x)+const)

    def __radd__(self, other):
        # Addition is commutative
        return self + other

    def __sub__(self,other):
        if callable(other):
            return self.__class__(lambda x,other=other,oldfunc=self : oldfunc(x)-other(x))
        else:
            return self.__class__(lambda x,const=other,oldfunc=self : oldfunc(x)-const)

    def __rsub__(self, other):
        if callable(other):
            return self.__class__(lambda x,other=other,oldfunc=self : other(x)-oldfunc(x))
        else:
            return self.__class__(lambda x,const=other,oldfunc=self : const-oldfunc(x))
        
    def __neg__(self):
        return self.__class__(lambda x,oldfunc=self : -oldfunc(x))

    def __mul__(self,other):
        if callable(other):
            return self.__class__(lambda x,other=other,oldfunc=self : oldfunc(x)*other(x))
        else:
            return self.__class__(lambda x,const=other,oldfunc=self : oldfunc(x)*const)
    
    def __rmul__(self,other):
        # Multiplication is commutative
        return self * other

    def __div__(self,other):
        if callable(other):
            return self.__class__(lambda x,other=other,oldfunc=self : oldfunc(x)/other(x))
        else:
            return self.__class__(lambda x,const=other,oldfunc=self : oldfunc(x)/const)
    
    def __rdiv__(self,other):
        if callable(other):
            return self.__class__(lambda x,other=other,oldfunc=self : other(x)/oldfunc(x))
        else:
            return self.__class__(lambda x,const=other,oldfunc=self : const/oldfunc(x))

class DisplacementField(VectorFunction):
    """A displacement field.

    Displacement fields are returned e.g. by the ``setup.dislocation``
    modules, they are vector fields (see ``VectorFunction``) and can be
    applied to a lattice using the ``apply_to()`` method.  The
    DisplacementField instance can also be used as a function taking
    an N x 3 array of positions and returning an N x 3 array of
    displacements.

    To create a displacement field, define a function taking an N x 3
    array of positions and returning an N x 3 array of displacements,
    then create the DisplacementField with the function as the sole
    argument.  A 'lambda' construct will often be useful.  For an
    example, see the source code of `setup.dislocation.ScrewDislocation`.
    
    """

    def apply_to(self, list, fixedbox=0, usepositions=None):
        """Apply the displacement field to an Atoms object.

        The atoms are modified to reflect the displacement field.

        Two optional arguments are possible.

        fixedpox (default: 0): Adjust the displacement field to keep
        the corners of the computational box fixed by adding a smooth
        displacement field to the specified field.

        usepositions (default: None): If given, it must be an array
        containing the positions of all atoms, to be used
        instead of the positions extracted from the list of atoms.  It
        can be useful if several displacement fields are to be applied
        to the same system, by giving the original positions as this
        argument the fields can be applied sequentially while
        obtaining an effect similar to adding the fields and then
        applying the sum.  This is mainly useful if the displacement
        field contains discontinuities, where a previously applied
        field (or dynamics) may cause some atoms to cross the
        discontinuity.
        """
        if fixedbox:
            box = list.get_cell()
            correction = _fixboxfield(self, box)
            field = self + correction
        else:
            field = self
        oldr = list.get_positions()
        if usepositions is None:
            usepositions = oldr
        else:
            if usepositions.shape != oldr.shape:
                raise ValueError("The optional argument usepositions has " +
                                   "the wrong shape: "+str(usepositions.shape))
        u = field(usepositions)
        list.set_positions(oldr + u)

def _fixboxfield(field, box):
    """Create a smooth displacement used to keep the computational box fixed.

    INTERNAL USE ONLY!  This function creates a smooth displacement field
    that can be added to an already existing field to keep the corners of the
    computational box fixed.
    """
    def cfield(r, invbox, k0, k1, k2, k3, k12, k13, k23, k123):
        """The correction field."""
        NewAxis = numpy.newaxis 
        (x1, x2, x3) = numpy.transpose(numpy.dot(r, invbox))
        return (k0 + k1*x1[:,NewAxis] + k2*x2[:,NewAxis] + k3*x3[:,NewAxis]
                + k12*(x1*x2)[:,NewAxis] + k13*(x1*x3)[:,NewAxis] +
                k23*(x2*x3)[:,NewAxis] + k123*(x1*x2*x3)[:,NewAxis])
    
    rs = numpy.zeros((2,2,2,3), float)
    for i in (0,1):
        for j in (0,1):
            for k in (0,1):
                rs[i,j,k,0] = i
                rs[i,j,k,1] = j
                rs[i,j,k,2] = k
    rs.shape = (8,3)                    # Reshape to a list of coordinates
    r = numpy.dot(rs, box) # Real space coordinates
    u = - field(r)                      # The desired corrections
    u.shape = (2,2,2,3)

    k0 = u[0,0,0]
    k1 = -u[0,0,0] + u[1,0,0]
    k2 = -u[0,0,0] + u[0,1,0]
    k3 = -u[0,0,0] + u[0,0,1]
    k12 = u[0,0,0] - u[1,0,0] -u[0,1,0] + u[1,1,0]
    k13 = u[0,0,0] - u[1,0,0] -u[0,0,1] + u[1,0,1]
    k23 = u[0,0,0] - u[0,1,0] -u[0,0,1] + u[0,1,1]
    k123 = (- u[0,0,0] + u[1,0,0] + u[0,1,0] + u[0,0,1]
            - u[1,1,0] - u[1,0,1] - u[0,1,1] + u[1,1,1])
    invbox = LinearAlgebra.inverse(box)

    return DisplacementField(lambda r, k0=k0, k1=k1, k2=k2, k3=k3, k12=k12,
                             k13=k13, k23=k23, k123=k123, invbox=invbox,
                             f=cfield:
                             f(r, invbox, k0, k1, k2, k3, k12, k13, k23, k123))




"""Test local crystal structure analysis (CNA and Coordination number)."""
from __future__ import print_function


#from asap3.analysis.localstructure import RestrictedCNA, CoordinationNumbers, FullCNA
from asap3 import *
from asap3.testtools import ReportTest
from asap3.mpi import world
from asap3.analysis.localstructure import RestrictedCNA, CoordinationNumbers

from asap3.optimize.fire import FIRE
from ase.lattice import bulk
import numpy as np

ismaster = world.rank == 0
isparallel = world.size != 1
cpulayout = [1, 1, world.size]
#set_verbose(1)

if ismaster:
    atoms = bulk('Cu')
    atoms = atoms.repeat((10, 10, 100))
    dx = 0.3 * np.sin(42 * np.arange(3 * len(atoms)))
    dx.shape = (-1, 3)
    atoms.set_array("perturbation", dx)
else:
    atoms = None
if isparallel:
    atoms = MakeParallelAtoms(atoms, cpulayout)    

print("Testing FIRE relaxation")
atoms.set_calculator(EMT())
e0 = atoms.get_potential_energy()
natoms = atoms.get_number_of_atoms()
ReportTest("Number of atoms", natoms, 10000, 0)
print("Potential energy before perturbation:", e0, e0/natoms)

atoms.set_positions(atoms.get_array("perturbation") + atoms.get_positions())
e1 = atoms.get_potential_energy()
print("Potential energy after perturbation:", e1, e1/natoms)

dyn = FIRE(atoms)
dyn.run(0.005)
e2 = atoms.get_potential_energy()
print("Potential energy after perturbation:", e2, e2/natoms)
ReportTest("Energy after relaxation", e2, e0, 0.02)
ReportTest("Number of FIRE steps", dyn.nsteps, 252, 5)

RestrictedCNA(atoms)
t = atoms.get_tags()
nwrong = np.not_equal(t, 0).sum()
ReportTest("Number of non-FCC atoms", nwrong, 0, 0)

ReportTest.Summary()



"""A calculator based on the OpenKIM project.

This module defines three object.

OpenKIMcalculator is a calculator class that interfaces to any OpenKIM V2 model.

OpenKIMparameters is a dictionary-like object that gives access to any
mutable parameters of the OpenKIM model, the object is created by
calling OpenKIMcalculator.get_parameters().

OpenKIMavailable is a function returning a list of available OpenKIM models.
"""
from __future__ import print_function, division

from ase.calculators.calculator import Calculator
from asap3 import _asap, AsapError
from asap3.Internal.ListOfElements import ListOfElements
from asap3.Internal.BuiltinPotentials import get_cell_heights, smallest_pbc_direction, AsapPotential
from ase.data import atomic_numbers, chemical_symbols
from ase.utils import basestring
import numpy as np
from copy import copy
import subprocess
import shutil
import os
try:
    from collections.abc import MutableMapping
except ImportError:
    # Python 2.7 ?
    from collections import MutableMapping
import numbers

class OpenKIMcalculator(_asap.OpenKIMcalculator, AsapPotential):
    """A calculator interfacing to the OpenKIM models.
    
    Parameters:
    
    name: The long name of the KIM Model.
    
    Optional parameters:
    
    atoms:  If set, set_atoms is called immediately.  Default: no atoms set.
                           
    stress:  Set to False to refrain from calculate the global virial, even if the model
             supports it.  Default:  True (calculate global virial / stress).
             
    stresses: As above, but for atomic virials / stresses.
    
    verbose:  Set to True to print additional info during neigborlist matching.
    """
    def __init__(self, name, atoms=None,
                 stress=True, stresses=True, verbose=False):
        _asap.OpenKIMcalculator.__init__(self, name, verbose)
        self.kimname = name
        self.atoms = None
        self.support_stress = stress
        self.support_stresses = stresses
        self.verbose = verbose
        if atoms is not None:
            self.set_atoms(atoms)
            
    def set_atoms(self, atoms, *args):
        """Set the atoms belonging to this calculator.
        
        The function set_atoms(atoms) is defined in C++, and just calls this
        Python function.  This ensures that this Python function is called
        regardless of whether set_atoms() is called from C++ or Python.
        """
        # Inform the calculator about the translation from atomic numbers to particle types.
        self.z_to_typecode = {}
        elements = ListOfElements(atoms)
        for e in elements:
            symbol = chemical_symbols[e]
            try:
                code = self.get_type_code(chemical_symbols[e])
            except AsapError:
                raise RuntimeError("The OpenKIM model '%s' does not support element Z=%i (%s)."
                                   % (self.kimname, e, chemical_symbols[e]))
            self.z_to_typecode[e] = code
            if self.verbose:
                print("Translation: Z = %i -> id = %i" % (e, code))
        self.set_translation(self.z_to_typecode)

        # Check the compute arguments required by the Model.
        computearguments = self._get_compute_arguments()
        requiredargs = ['numberOfParticles', 'particleSpeciesCodes', 'particleContributing',
                        'coordinates', 'partialEnergy', 'partialForces', 'GetNeighborList']
        for a in requiredargs:
            if computearguments[a] == 'notSupported':
                raise AsapError("The OpenKIM model '{}' does not support '{}'".format(self.kimname, a))
            del computearguments[a]  # Processed
        self.supported_calculations = {}
        for k in ('partialEnergy', 'partialForces'):
            self.supported_calculations[k] = True
        
        # partialParticleEnergy is optional, but we will calculate it if possible
        arg = 'partialParticleEnergy'
        self.supported_calculations[arg] = computearguments[arg] != 'notSupported'
        del computearguments[arg]
        # partialVirial and partialParticleVirial are allocated if they are required by the model, or if
        # they are optional and stress is requested.
        arg = 'partialVirial'
        self.supported_calculations[arg] = (
            computearguments[arg] == 'required' or
            (self.support_stress and computearguments[arg] == 'optional'))
        del computearguments[arg]
        arg = 'partialParticleVirial'
        self.supported_calculations[arg] = (
            computearguments[arg] == 'required' or
            (self.support_stress and computearguments[arg] == 'optional'))
        del computearguments[arg]
        # Make sure that the model does not require something extra.
        for a, s in computearguments.items():
            if s == 'required' or s == 'requiredByAPI':
                raise RuntimeError("OpenKIM model '{}' requires parameter '{}' (status: '{}')".format(
                    self.name, a, s))
        
        # Inform the calculator about which quantities should be allocated and calculated.
        for k,v in self.supported_calculations.items():
            self.please_support(k, v)

        # Find out which neighborlist type we got.  Check sanity and if ImageAtoms should be used.
        pbc = atoms.get_pbc()
        if self.verbose:
            print("Activating Image atoms (pbc = {})".format(str(pbc)))
        self._use_imageatoms()
        _asap.OpenKIMcalculator.set_atoms(self, atoms, *args)

    def get_supported_elements(self, user=True):
        """Get a list of supported elements (as chemical symbols).

        If the optional argument user=True (the default) then elements with
        names 'user01' to 'user20' may also be returned as supported.
        """
        supported = []
        candidates = chemical_symbols[1:]  # chemical_symbols[0] is 'X' and should be skipped.
        if user:
            candidates += ['user{:02d}'.format(x) for x in range(1,21)]
        for element in candidates:
            try:
                self.get_type_code(element)
            except AsapError:
                pass
            else:
                supported.append(element)
        return supported
    
    def get_parameters(self, kind='free', integers=[]):
        """Get the parameters of the model as a dictionary.
        
        The optional parameter 'kind' specifies which kind of parameters you get.
        It can take three values:
        'free' (the default).  You only get the "free" parameters, i.e. those that
            can be changed.
        'fixed': You only get the fixed parameters.
        'all': You get both free and fixed parameters.

        The optional argument 'integers' is a list of parameter names that are
        defined as integers in the parameter file.  This should be detected
        automatically once the KIM API specification is updated.
        
        WARNING: Currently using any other value than 'free' is dangerous, as the
        fixed parameter may be of unexpected types.  Once type-checking of parameters
        appear in the KIM API, accessing the fixed parameters will become safe, and 
        the default will change to 'all'.
        
        NOTE: The prefix PARAM_FREE_ or PARAM_FIXED_ is removed from the parameter 
        names.  If kind=='all', free parameters may shadow fixed parameters of the 
        same name.
        """
        if kind not in ['free', 'fixed', 'all']:
            raise ValueError("get_parameters: kind must be 'free', 'fixed' or 'all'.")
        result = {}
        if kind == 'fixed' or kind == 'all':
            prf = 'PARAM_FIXED_'
            for name in self.get_fixed_parameter_names():
                assert name.startswith(prf)
                shortname = name[len(prf):]
                if shortname in integers or name in integers:
                    value = self._get_parameter(name, is_integer=True)
                else:
                    value = self._get_parameter(name)
                result[shortname] = value
        if kind == 'free' or kind == 'all':
            prf = 'PARAM_FREE_'
            for name in self.get_free_parameter_names():
                assert name.startswith(prf)
                shortname = name[len(prf):]
                if shortname in integers or name in integers:
                    value = self._get_parameter(name, is_integer=True)
                else:
                    value = self._get_parameter(name)
                result[shortname] = value
        return result
    
    def set_parameters(self, **kwargs):
        """Set the free parameters of an OpenKIM model.
        
        Examples (these three are equivalent):
        
        model.set_parameters(cutoff = 3.7)
        
        model.set_parameters(PARAM_FREE_cutoff = 3.7)
        
        pars = {'cutoff': 3.7}
        model.set_parameters(**pars)
        
        Note that prefixing the parameter name with PARAM_FREE_ is optional.
        
        If a bad parameter specification is given, and this function fails,
        the model will be left in an undefined state.  That can be fixed by
        calling this function again, possibly without parameters.
        """
        prefix = 'PARAM_FREE_'
        allowed = self.get_free_parameter_names()
        for key, value in kwargs.items():
            if key.startswith(prefix):
                realkey = key
            else:
                realkey = prefix+key
            if realkey not in allowed:
                raise TypeError("Got an unexpected parameter name '{}'".format(key))
            if isinstance(value, float):
                self._set_parameter_scalar(realkey, value)
            elif isinstance(value, np.ndarray):
                self._set_parameter_array(realkey, value)
            else:
                raise TypeError("Unsupported value type - must be float or numpy array.")
        self._reinit()

    def get_name(self):
        return "asap3.OpenKIMcalculator(" + self.kimname + ")"

    def _get_parameters(self):
        return OpenKIMparameters(self)
    
    name = property(get_name, doc="Name of the potential")

    parameters = property(_get_parameters, doc="Published parameters of the OpenKIM model")

class OpenKIMparameters(MutableMapping):
    def __init__(self, model):
        self.model = model
        self.parameters = {}
        self.descriptions = {}
        for i, (name, typecode, size, description) in enumerate(model._get_parameter_names_types()):
            self.parameters[name] = (i, typecode, size)
            self.descriptions[name] = description

    def __len__(self):
        return len(self.parameters)

    def __getitem__(self, key):
        return self.model._get_parameter(*self.parameters[key])

    def __setitem__(self, key, value):
        index = self.parameters[key][0]
        if isinstance(value, numbers.Number):
            # A scalar
            self.model._set_parameter(index, value)
        else:
            # An array - flatten it.
            self.model._set_parameter(index, np.ravel(value)) 

    def __delitem__(self, key):
        raise RuntimeError("Cannot delete a parameter from an OpenKIM model.")
    
    def __iter__(self):
        for p in self.parameters:
            yield p
        
    def get_parameter(self, key, shape=None):
        p = self[key]
        if shape is None:
            return p
        elif isinstance(shape, tuple):
            return p.reshape(shape)
        shape = shape.lower()
        if shape == "square":
            size = int(np.round(np.sqrt(len(p))))
            if size*size == len(p):
                return p.reshape((size,size))
            else:
                raise ValueError("Cannot return parameter as a square matrix, its length is {}".format(len(p)))
        if shape == "upper triangular":
            shape = "ut"
        elif shape == "lower triangular":
            shape = "lt"
        if shape == "ut" or shape == "lt":
            size =  int(np.floor(np.sqrt(2*len(p))))
            if size * (size + 1) // 2 == len(p):
                result = np.zeros((size, size), dtype=p.dtype)
                if shape == "ut":
                    idx = np.triu_indices(size)
                else:
                    idx = np.tril_indices(size)
                result[idx] = p
                return result
            else:
                raise ValueError("Cannot return parameter as a triangular matrix ({}), its length is {}".format(shape, len(p)))
        else:
            raise ValueError("Unknown shape "+shape)
        
    def set_parameter(self, key, value, shape=None):
        if shape is None:
            self[key] = value
            return
        if len(value.shape) != 2 or value.shape[0] != value.shape[1]:
            raise ValueError("All supported shapes assume a square matrix")
        shape = shape.lower()
        if shape == "square":
            self[key] = value  # Flattened in __setitem__
        elif shape == "ut" or shape == "upper triangular":
            idx = np.triu_indices(len(value))
            self[key] = value[idx]
        elif shape == "lt" or shape == "lower triangular":
            idx = np.tril_indices(len(value))
            self[key] = value[idx]


def _get_openkim_model_string(verbose):
    exename = "kim-api-collections-info"
    args = ' models'
    sistername = 'kim-api-collections-management'

    # First try if the program is on the path.
    try:
        modelstring = subprocess.check_output(exename+args, shell=True, stderr=subprocess.STDOUT).decode('utf-8')
        return modelstring
    except subprocess.CalledProcessError:
        pass

    # Then look for it using pkg-config
    exepath = None
    try:
        prefix = subprocess.check_output("pkg-config libkim-api --variable=libexecdir", shell=True).decode('utf-8').strip()
    except subprocess.CalledProcessError:
        pass
    else:
        exepath = os.path.join(prefix, 'kim-api', exename)
        if verbose:
            print("Found using pkg-config:", exepath)

    # Finally, try to look for where kim-api-collections-management and then check ../libexec
    if not exepath and hasattr(shutil, 'which'):
        management = shutil.which(sistername)
        if management:
            prefix = os.path.dirname(os.path.dirname(management))
            exepath = os.path.join(prefix, 'libexec', 'kim-api', exename)
            if verbose:
                print("Found relative to", sistername, ":", exepath)
    if not exepath:
        raise RuntimeError("Cannot locate "+exename+" to generate list of KIM models.")

    if not os.access(exepath, os.R_OK):
        raise RuntimeError("Executable should be at "+exepath+" but is absent or not usable.")

    try:
        modelstring = subprocess.check_output(exepath+args, shell=True).decode('utf-8')
        return modelstring
    except subprocess.CalledProcessError:
        raise RuntimeError("Running "+exepath+" failed.")


def OpenKIMavailable(verbose=0):
    """Get the list of installed OpenKIM models.

    The list is obtained by locating and running the command line tool
    kim-api-collections-info
    """
    modelstring = _get_openkim_model_string(verbose)
    models = []
    for line in modelstring.split('\n'):
        if line:
            models.append(line.split()[1])
    return models


if __name__ == '__main__':
    from ase.lattice.cubic import FaceCenteredCubic
    atoms = FaceCenteredCubic(size=(10,10,10), symbol='Cu')
    print("Creating calculator")
    pot = OpenKIMcalculator('EMT_Asap_Standard_AlAgAuCuNiPdPt__MO_118428466217_000')
    print("Setting atoms")
    atoms.set_calculator(pot)
    print("Calculating energy")
    print(atoms.get_potential_energy())
    print(atoms.get_forces()[10:])
    print(atoms.get_stress())




#!/usr/bin/env python
"""
Tests that stresses are calculated correctly by Asap

Name: testStress.py

Description: Part of the Asap test suite.  Tests that stresses are
    calculated correctly by calculating various elastic constants from
    strain and stress and comparing them with the same constants
    calculated using strains and energies.  Can also be imported as a
    module, and used to test advanced calculation methods (QC,
    parallel).

Usage: python testStress.py

Expected result: Some elastic constants for Copper and Silver,
followed by the text 'ALL TESTS SUCCEEDED'.

The elastic constants are calculated by straining the crystal in
various modes, and fitting to the energies or the stresses.
Generally, the energies seems to be most sensitive to numerical noise,
and requires a rather large strain interval (1% or above), whereas the
stresses are much less sensitive to this.  On the other hand,
unlinearities influence the stress fits for large strain intervals.  A
strain interval of [-1%, 1%] is a good compromise, where both methods
work.

C11 and C12 calculated 'directly', i.e. in uniaxial strain, is
sensitive to the strain interval when using the energy to fit C11.
Fitting to the stresses work much better.

C11 and C12 can be calculated in an alternative way using a
volume-conserving deformation and fitting to the energies.

All of the above-mentioned calculations are performed.

"""
from __future__ import print_function

import sys
sys.path.append("..")
from StressModule import *
from ase import data, Atoms
from ase.lattice.cubic import FaceCenteredCubic
from ase.md.verlet import VelocityVerlet
from ase.md.langevin import Langevin
from asap3.mpi import world

debug = 0
if debug == 1:
    DebugOutput("parallelStress-%d.log", nomaster=True)
elif debug == 2:
    time.sleep(world.rank)
    print("PID:", os.getpid())
    time.sleep(20)

print_version(1)

#set_verbose(1)

ismaster = world.rank == 0
isparallel = world.size != 1
if world.size == 1:
    cpulayout = None
elif world.size == 2:
    cpulayout = [2,1,1]
elif world.size == 3:
    cpulayout = [1,3,1]
elif world.size == 4:
    cpulayout = [2,1,2]

defaultstrains = 0.01 * np.array((-1, -0.75, -0.5, -0.25, 0.0,
                                         0.25, 0.5, 0.75, 1.0))
                                       
book = {}
book['Cu'] = {"bookbulk": (134.3, 133.99),
              "bookc11": (168.3, 171.65),
              "bookc12": (122.1, 114.82),
              "bookc44": (75.7, 89.00)}

book['Copper_Rasmussen'] = {"bookbulk": (134.3, 140.77),
                            "bookc11": (168.3, 172.75),
                            "bookc12": (122.1, 124.32),
                            "bookc44": (75.7, 81.17)}

book['Ag'] = {"bookbulk": (103.8, 99.27),
              "bookc11": (124.0, 124.45),
              "bookc12": (93.7, 86.27),
              "bookc44": (46.12, 53.44)}


Copper = "Cu"
Silver = "Ag"
Copper_Rasm = "Copper_Rasmussen"
#for element in (Copper, Copper_Rasm, Silver):
for element in (Copper, Copper_Rasm, Silver):
    symb = element
    if (symb == "Copper_Rasmussen"):
        symbol = "Cu"
    else:
        symbol = symb
    print("")
    print("*** Running test on %s ***" % symb)
    z = data.atomic_numbers[symbol]
    latconst = data.reference_states[z]['a']
    initial = FaceCenteredCubic(directions=[[1,0,0],[0,1,0],[0,0,1]],
                                symbol=symbol, size=(10,10,10))
    ReportTest("Number of atoms", len(initial), 4000, 0)
    if ismaster:
        atoms = MakeParallelAtoms(initial, cpulayout)
    else:
        atoms = MakeParallelAtoms(None, cpulayout)
    if symb.find("Rasmussen") >= 0:
        print("Using the special EMT parameters by T. Rasmussen")
        atoms.set_calculator(EMT(EMTRasmussenParameters()))
    else:
        atoms.set_calculator(EMT())
    atoms.set_momenta(np.zeros((len(atoms),3), np.float))
    findlatticeconst(atoms, latconst)
    elasticconstants(atoms, symb, **book[symb])

ReportTest.Summary()



import math
import numpy as np


def multiple_factorization(n, length):
    '''
    finds all unique factorizations of n of a given length
    factors are in sorted order
    '''

    if length == 1:
        return []

    factorizations = []

    ulim0 = int(math.floor(math.sqrt(n + 1)))
    for i in range(1, ulim0 + 1):

        if n % i == 0:
            a = i
            b = n / i

            fa = multiple_factorization(a, length - 1)
            fb = multiple_factorization(b, length - 1)

            if not fa or not fb:
                factorizations += [[a, b]]

            for e in fa:
                factorizations += [[b] + e]
            for e in fb:
                factorizations += [[a] + e]

    return sorted([list(e) for e in set([tuple(sorted(e)) for e in factorizations])])

def calculate_optimal_cpu_setup(n, dim):
    '''
    finds optimal cpu setup for n processors with given sample dimensions
    '''

    if type(dim) == np.array:
        dim = dim.tolist()

    if len(dim) != 3:
        raise Exception("dim must be length 3")

    factorizations = multiple_factorization(n, 3)

    (dim, order) = zip(*sorted(zip(dim, range(3))))
    [len0, len1, len2] = [float(e) for e in dim]

    cubes = [(len0 / a, len1 / b, len2 / c) for (a, b, c) in factorizations]
    ratios = [(x*y*z) / (2*x*y + 2*y*z + 2*z*x) for (x, y, z) in cubes]

    best_fac = max([(e, bf) for e, bf in zip(ratios, factorizations)])[1]
    cpus = [e[1] for e in sorted(zip(order, best_fac))]
    return np.array(cpus)




#!/usr/bin/env python

#PBS -N HooverNPT
#PBS -m ae
#PBS -q long
#PBS -l nodes=1:opteron

from __future__ import print_function
from numpy import *
from asap3 import *
from asap3.md.npt import *
from asap3.md.langevin import *
from ase.md.velocitydistribution import *
from ase.lattice.cubic import FaceCenteredCubic
from asap3.testtools import ReportTest
import sys

T_goal = 300 # K
p_goal = 1 # GPa
bulk = 137 # Gpa
cp = 24.43 # J / (K * mol)
beta = 3 * 16.5e-6 # 1/K
#step1 = 10000
step1 = 10000
step2 = 10000

trajfile = "npt.bundle"

if len(sys.argv) == 1:
    out1 = "testNPT-1.out"
    out2 = "testNPT-2.out"
    ttime = 50
    ptime = 75
elif len(sys.argv) == 3:
    ttime = float(sys.argv[1])
    ptime = float(sys.argv[2])
    out1 = "testNPT-%d-%d-1.out" % (ttime, ptime)
    out2 = "testNPT-%d-%d-2.out" % (ttime, ptime)
else:
    raise RuntimeError("Expected zero or two parameter to the script!")

#ReportTest.Verbose()

atoms = FaceCenteredCubic(directions=((1,0,0), (0,1,0), (0,0,1)),
                          size=(15,15,15), symbol="Cu", pbc=True)
atoms.set_calculator(EMT())
print("Number of atoms:", len(atoms))

print("Heating to %d K using Langevin" % T_goal)
lgv = Langevin(atoms, 5 * units.fs, temperature=2*T_goal*units.kB,
               friction=0.01)
MaxwellBoltzmannDistribution(atoms, 0.9*T_goal*units.kB)

while atoms.get_kinetic_energy() < 1.5 * len(atoms) * T_goal * units.kB:
    lgv.run(10)
    T = atoms.get_kinetic_energy() / (1.5 * len(atoms) * units.kB)
    print("Temperature is now %.2f K" % (T,))
print("Desired temperature reached!")

lgv.set_temperature(T_goal*units.kB)

for i in range(4):
    lgv.run(100)
    s = atoms.get_stress()
    p = -(s[0] + s[1] + s[2])/3.0 / units.GPa
    T = atoms.get_kinetic_energy() / (1.5 * len(atoms) * units.kB)
    print("Pressure is %f GPa, desired pressure is %f GPa (T = %.2f K)" % (p, p_goal, T))
    dv = (p - p_goal) / bulk
    print("Adjusting volume by", dv)
    cell = atoms.get_cell()
    atoms.set_cell(cell * (1.0 + dv/3.0), scale_atoms=True)
    
T = atoms.get_kinetic_energy() / (1.5 * len(atoms) * units.kB)
print("Temperature is now %.2f K" % (T,))

dyn = NPT(atoms, 5 * units.fs, T_goal * units.kB, p_goal * units.GPa,
          ttime * units.fs, (ptime*units.fs)**2 * bulk * units.GPa)
traj = BundleTrajectory(trajfile, "w", atoms)
dyn.attach(traj, interval=50)

out = open(out1, "w")

temp = []
pres = []
vol = []
for i in xrange(step1):
    dyn.run(5)
    T = atoms.get_kinetic_energy() / (1.5 * len(atoms) * units.kB)
    s = atoms.get_stress() / units.GPa
    p = -(s[0] + s[1] + s[2])/3.0
    out.write("%5.2f  %7.5f      %7.5f %7.5f %7.5f %7.5f %7.5f %7.5f\n" %
              ((T, p)+tuple(s)))
    out.flush()
    cell = atoms.get_cell()
    v = cell[0,0] * cell[1,1] * cell[2,2]
    temp.append(T)
    pres.append(p)
    vol.append(v)
    if i % 10 == 0:
        print(i,"/",step1)

temp = array(temp[len(temp)/2:])
pres = array(pres[len(pres)/2:])
vol = array(vol[len(vol)/2:])

print("Average temperature: %.5f K  (standard deviation: %.5g K)" % (mean(temp), std(temp)))
ReportTest("Average temperature", mean(temp), T_goal,
           4*std(temp))
CV = cp * len(atoms) / 6.02205e23 / 1.60219e-19 # eV/K
expected_stddev_energy = sqrt(units.kB * T * T * CV)  # eV
expected_stddev_temp = sqrt(units.kB * T * T / CV) # K
#ReportTest("Standard deviation of temperature", std(temp),
#           expected_stddev_energy / (3 * len(atoms) * units.kB), 0.05, True)
ReportTest("Standard deviation of temperature",
           std(temp[len(temp)/2:]),
           expected_stddev_temp,
           0.5*expected_stddev_temp)

ReportTest("Average pressure", mean(pres), p_goal, 0.01, True)
expected_stddev_pressure = sqrt(units.kB * T * bulk * units.GPa / mean(vol))
expected_stddev_volume = sqrt(units.kB * T * mean(vol) / (bulk * units.GPa))
ReportTest("Standard deviation of pressure",
           std(pres[len(pres)/2:]),
           expected_stddev_pressure/units.GPa,
           0.2*expected_stddev_pressure/units.GPa)
ReportTest("Standard deviation of volume",
           std(vol),
           expected_stddev_volume,
           0.2*expected_stddev_volume)

dyn = NPT(atoms, 5 * units.fs, T_goal*units.kB,
          array([-2, -1, 0, 0, 0, 0])*p_goal*units.GPa,
          25*units.fs, (75*units.fs)**2 * bulk)

out = open(out2, "w")

stress = []
for i in xrange(step2):
    dyn.run(5)
    T = atoms.get_kinetic_energy() / (1.5 * len(atoms) * units.kB)
    s = atoms.get_stress() / units.GPa
    out.write("%5.2f      %7.5f %7.5f %7.5f %7.5f %7.5f %7.5f\n" %
              ((T, )+tuple(s)))
    out.flush()
    stress.append(s)
    if i % 10 == 0:
        print(i,"/",step2,"(part 2)")
    
stress = array(stress[len(stress)/2:])
s0 = mean(stress[:,0])
s1 = mean(stress[:,1])
s2 = mean(stress[:,2])
ReportTest("Stress component xx", s0, -2*p_goal, 0.05*p_goal)
ReportTest("Stress component yy", s1, -p_goal, 0.05*p_goal)
ReportTest("Stress component zz", s2, 0.0, 0.05*p_goal)
ReportTest.Summary()



from __future__ import print_function
from asap3 import *
from asap3.md.verlet import VelocityVerlet
from ase.lattice.cubic import FaceCenteredCubic
#from asap3.io.bundletrajectory import BundleTrajectory
from ase.io.bundletrajectory import BundleTrajectory
from ase.io import write
from numpy import *
import sys, os, time
from asap3.testtools import ReportTest

#DebugOutput("output.%d")
print_version(1)
delete = False #True

initial = FaceCenteredCubic(size=(10,10,10), symbol="Cu", pbc=(1,0,0))
initial.set_momenta(zeros((len(initial),3)))
initial.set_tags(arange(len(initial)))

atoms = initial.copy()
atoms.set_calculator(EMT())
print("Writing trajectory")
traj = BundleTrajectory("traj1.bundle", "w", atoms)
traj.write()
traj.close()





from __future__ import print_function
from asap3 import *
from asap3.mpi import world
from asap3.testtools import ReportTest
from ase.visualize import view
import numpy as np
import time
import os
import sys

N = 20
d = 3.0
w = 10.0

cpulayout = (2,1,1)
isparallel = world.size > 1
ismaster = world.rank == 0

debug = 0
if debug == 1:
    DebugOutput("force1D_%d.log", nomaster=True)
elif debug == 2:
    time.sleep(world.rank)
    print("PID:", os.getpid())
    time.sleep(20)
    
sys.stderr.write("\n\n\n\n")

def main():
    positions = np.zeros((N, 3))
    positions[:,0] = d * np.arange(N)
    if ismaster:
        atoms = Atoms("Cu%i" % (N,), positions=positions, cell=(N*d, w, w),
                      pbc=False)
        atoms.center()
    else:
        atoms = None
    if isparallel:
        atoms = MakeParallelAtoms(atoms, cpulayout)
        id = atoms.get_ids()
    else:
        id = np.arange(N)
    #view(atoms)
    
    atoms.set_calculator(EMT())
    f = atoms.get_forces()
    x = atoms.get_positions()[:,0]
    # time.sleep(world.rank)
    for i in range(len(atoms)):
        print(world.rank, x[i], f[i])
        
    # Now running actual tests - these are different on the two processors, but that should
    # not be a problem
    for i in range(len(f)):
        if id[i] > 1 and id[i] < N - 2:
            ftot = (f[i] * f[i]).sum()
            ReportTest("Force on atom %i" % (id[i],), ftot, 0.0, 1e-13)
    world.barrier()        
    ReportTest.Summary()
    
if world.size <= 2:
    main()
else:
    print("Skipping test when running on more than 2 processors.")
    


"""Asap builtin objects.

This module loads the modules from the compiled parts of Asap.  It
chooses between the parallel and serial version based on whether MPI
is active at runtime.
"""

__docformat__ = "restructuredtext en"

import sys

# Now import the relevant C module
if '_asap' in sys.builtin_module_names:
    # Build-in module (probably parallel simulation)
    import _asap
else:
    # Import depending on Python version.
    if sys.version_info[0] == 3:
        import _asap_p3 as _asap
    else:
        import _asap_p2 as _asap
        
parallelpossible = hasattr(_asap, "Communicator")
AsapError = _asap.AsapError
get_version = _asap.get_version
get_short_version = _asap.get_short_version
timing_results = _asap.timing_results

def set_verbose(v):
    """Set the verbosity level of asap."""
    _asap.verbose = v
    
